#!/usr/bin/env python
#coding:utf-8
import os
import ConfigParser
from toughradius.tools.shell import shell as sh
from toughradius.tools.secret import gen_secret

def find_config(conf_file=None):
    windows_dir = os.getenv("WINDIR") and os.path.join(os.getenv("WINDIR"),'radiusd.conf') or None
    cfgs = [
        conf_file,
        '/etc/radiusd.conf',
        '/var/toughradius/radiusd.conf',
        './radiusd.conf',
        '~/radiusd.conf',
        windows_dir
    ]
    config = ConfigParser.ConfigParser()
    flag = False
    for c in cfgs:
        if c and os.path.exists(c):
            config.read(c)
            flag = True
            break
   
    if not flag:
        raise Exception('no config file')
                
    return config
    
    

def setup_config():
    sh.info("set config...")
    config_path = sh.read('set your config file path,[ /etc/radiusd.conf ]') or '/etc/radiusd.conf'
    config = ConfigParser.RawConfigParser()
    sh.info("set default option")
    config.set('DEFAULT', 'debug', (sh.read("set debug [0/1] [0]:") or '0') )
    config.set('DEFAULT', 'tz', (sh.read("time zone [ CST-8 ]:") or 'CST-8') )
    config.set('DEFAULT','secret',gen_secret(32))
    
    sh.info("set database option")
    config.add_section('database')
    config.set('database','dbtype', (sh.read("database type [sqlite]:") or 'sqlite' ))
    config.set('database','dburl',( sh.read("database url [sqlite:////tmp/toughradius.sqlite3]:") or 'sqlite:////tmp/toughradius.sqlite3' ))
    config.set('database','echo',(sh.read("database echo sql [false]:") or 'false' ))
    config.set('database','pool_size',(sh.read("database pool_size [30]:") or '30' ))
    config.set('database','pool_recycle',(sh.read("database pool_recycle(second) [300]:") or '300' ))
    
    sh.info("set radiusd option")
    config.add_section('radiusd')
    config.set('radiusd','authport',(sh.read("radiusd authport [1812]:") or '1812'))
    config.set('radiusd','acctport',(sh.read("radiusd acctport [1813]:") or '1813'))
    config.set('radiusd','adminport',(sh.read("radiusd adminport [1815]:") or '1815'))
    config.set('radiusd','cache_timeout',(sh.read("radiusd cache_timeout (second) [600]:") or '600'))
    config.set('radiusd', 'logfile', (sh.read("log file [ /var/log/radiusd.log ]:") or '/var/log/radiusd.log') )
    
    sh.info("set admin option")
    config.add_section('admin')
    config.set('admin','port',(sh.read("admin http port [1816]:") or '1816'))
    config.set('admin', 'logfile', (sh.read("log file [ /var/log/admin.log ]:") or '/var/log/admin.log') )
    
    sh.info("set customer option")
    config.add_section('customer')
    config.set('customer','port',(sh.read("customer http port [1817]:") or '1817'))
    config.set('customer', 'logfile', (sh.read("log file [ /var/log/customer.log ]:") or '/var/log/customer.log') )
    
    with open(config_path,'wb') as configfile:
        config.write(configfile)
        sh.succ("config save to %s"%config_path)

def echo_radiusd_cnf():
    return '''[DEFAULT]
debug = 0
tz = CST-8
secret = %s

[database]
dbtype = sqlite
dburl = sqlite:////tmp/toughradius.sqlite3
echo = false
# dbtype = mysql
# dburl = mysql://root:root@127.0.0.1/toughradius?charset=utf8
# pool_size = 60
# pool_recycle = 300

[radiusd]
acctport = 1813
adminport = 1815
authport = 1812
cache_timeout = 600
logfile = /var/log/radiusd.log

[admin]
port = 1816
logfile = /var/log/admin.log

[customer]
port = 1817
logfile = /var/log/customer.log
'''%gen_secret(32)



def echo_app_tac(app):
    if app == 'radiusd':
        return '''from autobahn.twisted import choosereactor
choosereactor.install_optimal_reactor(True)
from twisted.application import service, internet
from toughradius.tools import config
from toughradius.tools.dbengine import DBEngine
from toughradius.radiusd import server
application = service.Application("ToughRADIUS Radiusd Application")
config = config.find_config()
service = server.run(config,DBEngine(config).get_engine(),True)
service.setServiceParent(application)'''
    elif app == 'admin':
        return '''from autobahn.twisted import choosereactor
choosereactor.install_optimal_reactor(True)
from twisted.application import service, internet
from toughradius.tools import config
from toughradius.tools.dbengine import DBEngine
from toughradius.console import admin_app
application = service.Application("ToughRADIUS Admin Application")
config = config.find_config()
service = admin_app.run(config,DBEngine(config).get_engine(),True)
service.setServiceParent(application)'''
    elif app == 'customer':
        return '''from autobahn.twisted import choosereactor
choosereactor.install_optimal_reactor(True)
from twisted.application import service, internet
from toughradius.tools import config
from toughradius.tools.dbengine import DBEngine
from toughradius.console import customer_app
application = service.Application("ToughRADIUS Customer Application")
config = config.find_config()
service = customer_app.run(config,DBEngine(config).get_engine(),True)
service.setServiceParent(application)'''
    elif app == 'standalone':
        return '''from autobahn.twisted import choosereactor
choosereactor.install_optimal_reactor(True)
from twisted.application import service, internet
from toughradius.tools import config
from toughradius.tools.dbengine import get_engine
from toughradius.console import admin_app
from toughradius.console import customer_app
from toughradius.radiusd import server
application = service.Application("ToughRADIUS Standalone Application")
config = config.find_config()
db_engine = get_engine(config)
service = server.run(config,db_engine,True)
admin_app.run(config,db_engine,True).setServiceParent(service)
customer_app.run(config,db_engine,True).setServiceParent(service)
service.setServiceParent(application)'''


# packet.py
#
# Copyright 2002-2005,2007 Wichert Akkerman <wichert@wiggy.net>
#
# A RADIUS packet as defined in RFC 2138


import struct
import random
try:
    import hashlib
    md5_constructor = hashlib.md5
except ImportError:
    # BBB for python 2.4
    import md5
    md5_constructor = md5.new
import six
import tools

# Packet codes
AccessRequest = 1
AccessAccept = 2
AccessReject = 3
AccountingRequest = 4
AccountingResponse = 5
AccessChallenge = 11
StatusServer = 12
StatusClient = 13
DisconnectRequest = 40
DisconnectACK = 41
DisconnectNAK = 42
CoARequest = 43
CoAACK = 44
CoANAK = 45

# Use cryptographic-safe random generator as provided by the OS.
random_generator = random.SystemRandom()

# Current ID
CurrentID = random_generator.randrange(1, 255)


class PacketError(Exception):
    pass


class Packet(dict):
    """Packet acts like a standard python map to provide simple access
    to the RADIUS attributes. Since RADIUS allows for repeated
    attributes the value will always be a sequence. pyrad makes sure
    to preserve the ordering when encoding and decoding packets.

    There are two ways to use the map intereface: if attribute
    names are used pyrad take care of en-/decoding data. If
    the attribute type number (or a vendor ID/attribute type
    tuple for vendor attributes) is used you work with the
    raw data.

    Normally you will not use this class directly, but one of the
    :obj:`AuthPacket` or :obj:`AcctPacket` classes.
    """

    def __init__(self, code=0, id=None, secret=six.b(''), authenticator=None,
            **attributes):
        """Constructor

        :param dict:   RADIUS dictionary
        :type dict:    pyrad.dictionary.Dictionary class
        :param secret: secret needed to communicate with a RADIUS server
        :type secret:  string
        :param id:     packet identifaction number
        :type id:      integer (8 bits)
        :param code:   packet type code
        :type code:    integer (8bits)
        :param packet: raw packet to decode
        :type packet:  string
        """
        dict.__init__(self)
        self.code = code
        if id is not None:
            self.id = id
        else:
            self.id = CreateID()
        if not isinstance(secret, six.binary_type):
            raise TypeError('secret must be a binary string')
        self.secret = secret
        if authenticator is not None and \
                not isinstance(authenticator, six.binary_type):
                    raise TypeError('authenticator must be a binary string')
        self.authenticator = authenticator

        if 'dict' in attributes:
            self.dict = attributes['dict']

        if 'packet' in attributes:
            self.DecodePacket(attributes['packet'])

        for (key, value) in attributes.items():
            if key in ['dict', 'fd', 'packet']:
                continue
            key = key.replace('_', '-')
            self.AddAttribute(key, value)

    def CreateReply(self, **attributes):
        """Create a new packet as a reply to this one. This method
        makes sure the authenticator and secret are copied over
        to the new instance.
        """
        return Packet(id=self.id, secret=self.secret,
                      authenticator=self.authenticator, dict=self.dict,
                      **attributes)

    def _DecodeValue(self, attr, value):
        if attr.values.HasBackward(value):
            return attr.values.GetBackward(value)
        else:
            return tools.DecodeAttr(attr.type, value)

    def _EncodeValue(self, attr, value):
        if attr.values.HasForward(value):
            return attr.values.GetForward(value)
        else:
            return tools.EncodeAttr(attr.type, value)

    def _EncodeKeyValues(self, key, values):
        if not isinstance(key, str):
            return (key, values)

        attr = self.dict.attributes[key]
        if attr.vendor:
            key = (self.dict.vendors.GetForward(attr.vendor), attr.code)
        else:
            key = attr.code

        return (key, [self._EncodeValue(attr, v) for v in values])

    def _EncodeKey(self, key):
        if not isinstance(key, str):
            return key

        attr = self.dict.attributes[key]
        if attr.vendor:
            return (self.dict.vendors.GetForward(attr.vendor), attr.code)
        else:
            return attr.code

    def _DecodeKey(self, key):
        """Turn a key into a string if possible"""

        if self.dict.attrindex.HasBackward(key):
            return self.dict.attrindex.GetBackward(key)
        return key

    def AddAttribute(self, key, value):
        """Add an attribute to the packet.

        :param key:   attribute name or identification
        :type key:    string, attribute code or (vendor code, attribute code)
                      tuple
        :param value: value
        :type value:  depends on type of attribute
        """
        (key, value) = self._EncodeKeyValues(key, [value])
        value = value[0]

        self.setdefault(key, []).append(value)

    def __getitem__(self, key):
        if not isinstance(key, six.string_types):
            return dict.__getitem__(self, key)

        values = dict.__getitem__(self, self._EncodeKey(key))
        attr = self.dict.attributes[key]
        res = []
        for v in values:
            res.append(self._DecodeValue(attr, v))
        return res

    def __contains__(self, key):
        try:
            return dict.__contains__(self, self._EncodeKey(key))
        except KeyError:
            return False

    has_key = __contains__

    def __delitem__(self, key):
        dict.__delitem__(self, self._EncodeKey(key))

    def __setitem__(self, key, item):
        if isinstance(key, six.string_types):
            (key, item) = self._EncodeKeyValues(key, [item])
            dict.__setitem__(self, key, item)
        else:
            assert isinstance(item, list)
            dict.__setitem__(self, key, item)

    def keys(self):
        return [self._DecodeKey(key) for key in dict.keys(self)]

    @staticmethod
    def CreateAuthenticator():
        """Create a packet autenticator. All RADIUS packets contain a sixteen
        byte authenticator which is used to authenticate replies from the
        RADIUS server and in the password hiding algorithm. This function
        returns a suitable random string that can be used as an authenticator.

        :return: valid packet authenticator
        :rtype: binary string
        """

        data = []
        for i in range(16):
            data.append(random_generator.randrange(0, 256))
        if six.PY3:
            return bytes(data)
        else:
            return ''.join(chr(b) for b in data)

    def CreateID(self):
        """Create a packet ID.  All RADIUS requests have a ID which is used to
        identify a request. This is used to detect retries and replay attacks.
        This function returns a suitable random number that can be used as ID.

        :return: ID number
        :rtype:  integer

        """
        return random_generator.randrange(0, 256)

    def ReplyPacket(self):
        """Create a ready-to-transmit authentication reply packet.
        Returns a RADIUS packet which can be directly transmitted
        to a RADIUS server. This differs with Packet() in how
        the authenticator is calculated.

        :return: raw packet
        :rtype:  string
        """
        assert(self.authenticator)
        assert(self.secret is not None)

        attr = self._PktEncodeAttributes()
        header = struct.pack('!BBH', self.code, self.id, (20 + len(attr)))

        authenticator = md5_constructor(header[0:4] + self.authenticator
                              + attr + self.secret).digest()
        return header + authenticator + attr

    def VerifyReply(self, reply, rawreply=None):
        if reply.id != self.id:
            return False

        if rawreply is None:
            rawreply = reply.ReplyPacket()

        hash = md5_constructor(rawreply[0:4] + self.authenticator +
                     rawreply[20:] + self.secret).digest()

        if hash != rawreply[4:20]:
            return False
        return True

    def _PktEncodeAttribute(self, key, value):
        if isinstance(key, tuple):
            value = struct.pack('!L', key[0]) + \
                self._PktEncodeAttribute(key[1], value)
            key = 26

        return struct.pack('!BB', key, (len(value) + 2)) + value

    def _PktEncodeAttributes(self):
        result = six.b('')
        for (code, datalst) in self.items():
            for data in datalst:
                result += self._PktEncodeAttribute(code, data)

        return result

    def _PktDecodeVendorAttribute(self, data):
        # Check if this packet is long enough to be in the
        # RFC2865 recommended form
        if len(data) < 6:
            return (26, data)

        (vendor, type, length) = struct.unpack('!LBB', data[:6])[0:3]
        # Another sanity check
        if len(data) != length + 4:
            return (26, data)

        return ((vendor, type), data[6:])

    def DecodePacket(self, packet):
        """Initialize the object from raw packet data.  Decode a packet as
        received from the network and decode it.

        :param packet: raw packet
        :type packet:  string"""

        try:
            (self.code, self.id, length, self.authenticator) = \
                    struct.unpack('!BBH16s', packet[0:20])
        except struct.error:
            raise PacketError('Packet header is corrupt')
        if len(packet) != length:
            raise PacketError('Packet has invalid length')
        if length > 8192:
            raise PacketError('Packet length is too long (%d)' % length)

        self.clear()

        packet = packet[20:]
        while packet:
            try:
                (key, attrlen) = struct.unpack('!BB', packet[0:2])
            except struct.error:
                raise PacketError('Attribute header is corrupt')

            if attrlen < 2:
                raise PacketError(
                        'Attribute length is too small (%d)' % attrlen)

            value = packet[2:attrlen]
            if key == 26:
                (key, value) = self._PktDecodeVendorAttribute(value)

            self.setdefault(key, []).append(value)
            packet = packet[attrlen:]


class AuthPacket(Packet):
    def __init__(self, code=AccessRequest, id=None, secret=six.b(''),
            authenticator=None, **attributes):
        """Constructor

        :param code:   packet type code
        :type code:    integer (8bits)
        :param id:     packet identifaction number
        :type id:      integer (8 bits)
        :param secret: secret needed to communicate with a RADIUS server
        :type secret:  string

        :param dict:   RADIUS dictionary
        :type dict:    pyrad.dictionary.Dictionary class

        :param packet: raw packet to decode
        :type packet:  string
        """
        Packet.__init__(self, code, id, secret, authenticator, **attributes)

    def CreateReply(self, **attributes):
        """Create a new packet as a reply to this one. This method
        makes sure the authenticator and secret are copied over
        to the new instance.
        """
        return AuthPacket(AccessAccept, self.id,
            self.secret, self.authenticator, dict=self.dict,
            **attributes)

    def RequestPacket(self):
        """Create a ready-to-transmit authentication request packet.
        Return a RADIUS packet which can be directly transmitted
        to a RADIUS server.

        :return: raw packet
        :rtype:  string
        """
        attr = self._PktEncodeAttributes()

        if self.authenticator is None:
            self.authenticator = self.CreateAuthenticator()

        if self.id is None:
            self.id = self.CreateID()

        header = struct.pack('!BBH16s', self.code, self.id,
            (20 + len(attr)), self.authenticator)

        return header + attr

    def PwDecrypt(self, password):
        """Unobfuscate a RADIUS password. RADIUS hides passwords in packets by
        using an algorithm based on the MD5 hash of the packet authenticator
        and RADIUS secret. This function reverses the obfuscation process.

        :param password: obfuscated form of password
        :type password:  binary string
        :return:         plaintext password
        :rtype:          unicode string
        """
        buf = password
        pw = six.b('')

        last = self.authenticator
        while buf:
            hash = md5_constructor(self.secret + last).digest()
            if six.PY3:
                for i in range(16):
                    pw += bytes((hash[i] ^ buf[i],))
            else:
                for i in range(16):
                    pw += chr(ord(hash[i]) ^ ord(buf[i]))

            (last, buf) = (buf[:16], buf[16:])

        while pw.endswith(six.b('\x00')):
            pw = pw[:-1]

        return pw.decode('utf-8')

    def PwCrypt(self, password):
        """Obfuscate password.
        RADIUS hides passwords in packets by using an algorithm
        based on the MD5 hash of the packet authenticator and RADIUS
        secret. If no authenticator has been set before calling PwCrypt
        one is created automatically. Changing the authenticator after
        setting a password that has been encrypted using this function
        will not work.

        :param password: plaintext password
        :type password:  unicode stringn
        :return:         obfuscated version of the password
        :rtype:          binary string
        """
        if self.authenticator is None:
            self.authenticator = self.CreateAuthenticator()

        if isinstance(password, six.text_type):
            password = password.encode('utf-8')

        buf = password
        if len(password) % 16 != 0:
            buf += six.b('\x00') * (16 - (len(password) % 16))

        hash = md5_constructor(self.secret + self.authenticator).digest()
        result = six.b('')

        last = self.authenticator
        while buf:
            hash = md5_constructor(self.secret + last).digest()
            if six.PY3:
                for i in range(16):
                    result += bytes((hash[i] ^ buf[i],))
            else:
                for i in range(16):
                    result += chr(ord(hash[i]) ^ ord(buf[i]))

            last = result[-16:]
            buf = buf[16:]

        return result


class AcctPacket(Packet):
    """RADIUS accounting packets. This class is a specialization
    of the generic :obj:`Packet` class for accounting packets.
    """

    def __init__(self, code=AccountingRequest, id=None, secret=six.b(''),
            authenticator=None, **attributes):
        """Constructor

        :param dict:   RADIUS dictionary
        :type dict:    pyrad.dictionary.Dictionary class
        :param secret: secret needed to communicate with a RADIUS server
        :type secret:  string
        :param id:     packet identifaction number
        :type id:      integer (8 bits)
        :param code:   packet type code
        :type code:    integer (8bits)
        :param packet: raw packet to decode
        :type packet:  string
        """
        Packet.__init__(self, code, id, secret, authenticator, **attributes)
        if 'packet' in attributes:
            self.raw_packet = attributes['packet']

    def CreateReply(self, **attributes):
        """Create a new packet as a reply to this one. This method
        makes sure the authenticator and secret are copied over
        to the new instance.
        """
        return AcctPacket(AccountingResponse, self.id,
            self.secret, self.authenticator, dict=self.dict,
            **attributes)

    def VerifyAcctRequest(self):
        """Verify request authenticator.

        :return: True if verification failed else False
        :rtype: boolean
        """
        assert(self.raw_packet)
        hash = md5_constructor(self.raw_packet[0:4] + 16 * six.b('\x00') +
                self.raw_packet[20:] + self.secret).digest()
        return hash == self.authenticator

    def RequestPacket(self):
        """Create a ready-to-transmit authentication request packet.
        Return a RADIUS packet which can be directly transmitted
        to a RADIUS server.

        :return: raw packet
        :rtype:  string
        """

        attr = self._PktEncodeAttributes()

        if self.id is None:
            self.id = self.CreateID()

        header = struct.pack('!BBH', self.code, self.id, (20 + len(attr)))
        self.authenticator = md5_constructor(header[0:4] + 16 * six.b('\x00') + attr
            + self.secret).digest()
        return header + self.authenticator + attr


class CoAPacket(Packet):
    """RADIUS CoA packets. This class is a specialization
    of the generic :obj:`Packet` class for CoA packets.
    """

    def __init__(self, code=CoARequest, id=None, secret=six.b(''),
            authenticator=None, **attributes):
        """Constructor
        :param dict:   RADIUS dictionary
        :type dict:    pyrad.dictionary.Dictionary class
        :param secret: secret needed to communicate with a RADIUS server
        :type secret:  string
        :param id:     packet identifaction number
        :type id:      integer (8 bits)
        :param code:   packet type code
        :type code:    integer (8bits)
        :param packet: raw packet to decode
        :type packet:  string
        """
        Packet.__init__(self, code, id, secret, authenticator, **attributes)
        if 'packet' in attributes:
            self.raw_packet = attributes['packet']

    def CreateReply(self, **attributes):
        """Create a new packet as a reply to this one. This method
        makes sure the authenticator and secret are copied over
        to the new instance.
        """
        return CoAPacket(CoAACK, self.id,
            self.secret, self.authenticator, dict=self.dict,
            **attributes)

    def VerifyCoARequest(self):
        """Verify request authenticator.
        :return: True if verification failed else False
        :rtype: boolean
        """
        assert(self.raw_packet)
        hash = md5_constructor(self.raw_packet[0:4] + 16 * six.b('\x00') +
                self.raw_packet[20:] + self.secret).digest()
        return hash == self.authenticator

    def RequestPacket(self):
        """Create a ready-to-transmit CoA request packet.
        Return a RADIUS packet which can be directly transmitted
        to a RADIUS server.
        :return: raw packet
        :rtype:  string
        """

        attr = self._PktEncodeAttributes()

        if self.id is None:
            self.id = self.CreateID()

        header = struct.pack('!BBH', self.code, self.id, (20 + len(attr)))
        self.authenticator = md5_constructor(header[0:4] + 16 * six.b('\x00') + attr
            + self.secret).digest()
        return header + self.authenticator + attr        


def CreateID():
    """Generate a packet ID.

    :return: packet ID
    :rtype:  8 bit integer
    """
    global CurrentID

    CurrentID = (CurrentID + 1) % 256
    return CurrentID



#coding=utf-8
from twisted.python import log
from toughradius.radiusd import plugins
import logging

class Middleware():
    _plugin_modules = {}
    _plugin_loaded = False

    @classmethod
    def load_plugins(cls):
        if cls._plugin_loaded:
            return
        for name in plugins.__all__:
            try:
                __import__('toughradius.radiusd.plugins',globals(),locals(), [name])
                cls.add_plugin(name,getattr(plugins,name))
                log.msg('Plugin %s loaded success.' % name,level=logging.INFO)
            except Exception as err:
                log.err(err,'Fail to load plugin %s' % (name),level=logging.INFO)
        cls._plugin_loaded = True

    @classmethod
    def add_plugin(cls,name,plugin):
        if not hasattr(plugin, 'process'):
            log.err('Plugin %s has no method named process, ignore it',level=logging.INFO)
            return False
        cls._plugin_modules[name] = plugin
        return True

    def process(self,name,**kwargs):
        if name not in self._plugin_modules:
            raise Exception('Plugin %s not find. ' % (name))
        try:
            # log.msg('Plugin %s match.'%name,level=logging.DEBUG)
            return self._plugin_modules[name].process(**kwargs)
        except Exception as err:
            log.err(err,'Plugin %s failed to process.' % name,level=logging.INFO)
            import traceback
            traceback.print_exc()
            return False
             

#first init plugins
Middleware.load_plugins()

if __name__ == '__main__':
    pass



#!/usr/bin/env python
#coding=utf-8
from twisted.python import log
from toughradius.radiusd.pyrad import packet
from toughradius.radiusd.settings import *
from toughradius.radiusd import utils
import logging
import datetime
import decimal

decimal.getcontext().prec = 32
decimal.getcontext().rounding = decimal.ROUND_UP

def send_dm(coa_clients,online):
    try:
        coa_client = coa_clients.get(online['nas_addr'])
        attrs = {
            'User-Name' : online['account_number'],
            'Acct-Session-Id' : online['acct_session_id'],
            'NAS-IP-Address' : online['nas_addr'],
            'Framed-IP-Address' : online['framed_ipaddr']
        }
        dmeq = coa_client.createDisconnectPacket(**attrs)
        coa_client.sendCoA(dmeq)
    except:
        log.err('send dm error')

def process(req=None,user=None,radiusd=None,**kwargs):
    if req.get_acct_status_type() not in (STATUS_TYPE_UPDATE,STATUS_TYPE_STOP):
        return   
    
    runstat=radiusd.runstat
    coa_clients=radiusd.coa_clients
    store = radiusd.store
        
    online = store.get_online(req.get_nas_addr(),req.get_acct_sessionid())  
    if not online:
        return
    
    online = dict(online)

    product = store.get_product(user['product_id'])
    if not product or product['product_policy'] not in (PPTimes,BOTimes,PPFlow,BOFlows):
        online['billing_times'] = req.get_acct_sessiontime()
        online['input_total'] = req.get_input_total()
        online['output_total'] = req.get_output_total()
        store.update_online(online)
        return

    def process_pptimes():
        # 预付费时长
        log.msg('%s > Prepaid long time billing '%req.get_user_name(),level=logging.INFO)
        user_balance = store.get_user_balance(user['account_number'])
        sessiontime = decimal.Decimal(req.get_acct_sessiontime())
        billing_times = decimal.Decimal(online['billing_times'])
        acct_times = sessiontime - billing_times
        fee_price = decimal.Decimal(product['fee_price'])
        usedfee = acct_times/decimal.Decimal(3600) * fee_price
        usedfee = actual_fee = int(usedfee.to_integral_value())
        balance = user_balance - usedfee
        
        if balance < 0 :  
            balance = 0
            actual_fee = user_balance
            
        store.update_billing(utils.Storage(
            account_number = online['account_number'],
            nas_addr = online['nas_addr'],
            acct_session_id = online['acct_session_id'],
            acct_start_time = online['acct_start_time'],
            acct_session_time = req.get_acct_sessiontime(),
            input_total = req.get_input_total(),
            output_total = req.get_output_total(),
            acct_times = int(acct_times.to_integral_value()),
            acct_flows = 0,
            acct_fee = usedfee,
            actual_fee = actual_fee,
            balance = balance,
            time_length = 0,
            flow_length = 0,
            is_deduct = 1,
            create_time = datetime.datetime.now().strftime( "%Y-%m-%d %H:%M:%S")
        ))
        
        if balance == 0 :
            send_dm(coa_clients,online)
        
    def process_botimes():
        #买断时长
        log.msg('%s > Buyout long time billing '%req.get_user_name(),level=logging.INFO)
        time_length = store.get_user_time_length(user['account_number'])
        sessiontime = req.get_acct_sessiontime()
        billing_times = online['billing_times']
        acct_times = sessiontime - billing_times
        user_time_length = time_length - acct_times
        if user_time_length < 0 :
            user_time_length = 0

        store.update_billing(utils.Storage(
            account_number = online['account_number'],
            nas_addr = online['nas_addr'],
            acct_session_id = online['acct_session_id'],
            acct_start_time = online['acct_start_time'],
            acct_session_time = req.get_acct_sessiontime(),
            input_total = req.get_input_total(),
            output_total = req.get_output_total(),
            acct_times = acct_times,
            acct_flows = 0,
            acct_fee = 0,
            actual_fee = 0,
            balance = 0,
            time_length = user_time_length,
            flow_length = 0,
            is_deduct = 1,
            create_time = datetime.datetime.now().strftime( "%Y-%m-%d %H:%M:%S")
        ),time_length=user_time_length)
    
        if user_time_length == 0 :
            send_dm(coa_clients,online)
        
    def process_ppflows():
        #预付费流量
        log.msg('%s > Prepaid flow billing '%req.get_user_name(),level=logging.INFO)
        user_balance = store.get_user_balance(user['account_number'])
        output_total = decimal.Decimal(req.get_output_total())
        billing_output_total = decimal.Decimal(online['output_total'])
        acct_flows = output_total - billing_output_total
        fee_price = decimal.Decimal(product['fee_price'])
        usedfee = acct_flows/decimal.Decimal(1024) * fee_price
        usedfee = actual_fee = int(usedfee.to_integral_value())
        balance = user_balance - usedfee
        
        if balance < 0 :  
            balance = 0
            actual_fee = user_balance
            
        store.update_billing(utils.Storage(
            account_number = online['account_number'],
            nas_addr = online['nas_addr'],
            acct_session_id = online['acct_session_id'],
            acct_start_time = online['acct_start_time'],
            acct_session_time = req.get_acct_sessiontime(),
            input_total = req.get_input_total(),
            output_total = req.get_output_total(),
            acct_times = 0,
            acct_flows = int(acct_flows.to_integral_value()),
            acct_fee = usedfee,
            actual_fee = actual_fee,
            balance = balance,
            time_length = 0,
            flow_length = 0,
            is_deduct = 1,
            create_time = datetime.datetime.now().strftime( "%Y-%m-%d %H:%M:%S")
        ))
        
        if balance == 0 :  
            send_dm(coa_clients,online)
        
    def process_boflows():
        #买断流量
        log.msg('%s > Buyout flow billing '%req.get_user_name(),level=logging.INFO)
        flow_length = store.get_user_flow_length(user['account_number'])
        output_total = req.get_output_total()
        billing_output_total = online['output_total']
        acct_flows = output_total - billing_output_total
        user_flow_length = flow_length - acct_flows
        if user_flow_length < 0 :
            user_flow_length = 0
            send_dm(coa_clients,online)
            
        store.update_billing(utils.Storage(
            account_number = online['account_number'],
            nas_addr = online['nas_addr'],
            acct_session_id = online['acct_session_id'],
            acct_start_time = online['acct_start_time'],
            acct_session_time = req.get_acct_sessiontime(),
            input_total = req.get_input_total(),
            output_total = req.get_output_total(),
            acct_times = 0,
            acct_flows = acct_flows,
            acct_fee = 0,
            actual_fee = 0,
            balance = 0,
            time_length = 0,
            flow_length = user_flow_length,
            is_deduct = 1,
            create_time = datetime.datetime.now().strftime( "%Y-%m-%d %H:%M:%S")
        ),flow_length=user_flow_length)
        
        if user_flow_length == 0 :
            send_dm(coa_clients,online)
    
    process_funcs = {
        PPTimes:process_pptimes,
        BOTimes:process_botimes,
        PPFlow:process_ppflows,
        BOFlows:process_boflows
    }
    
    process_funcs[product['product_policy']]()


# client.py
#
# Copyright 2002-2007 Wichert Akkerman <wichert@wiggy.net>

__docformat__ = "epytext en"

import select
import socket
import time
import six
import host
import packet


class Timeout(Exception):
    """Simple exception class which is raised when a timeout occurs
    while waiting for a RADIUS server to respond."""


class Client(host.Host):
    """Basic RADIUS client.
    This class implements a basic RADIUS client. It can send requests
    to a RADIUS server, taking care of timeouts and retries, and
    validate its replies.

    :ivar retries: number of times to retry sending a RADIUS request
    :type retries: integer
    :ivar timeout: number of seconds to wait for an answer
    :type timeout: integer
    """
    def __init__(self, server, authport=1812, acctport=1813,
            secret=six.b(''), dict=None):

        """Constructor.

        :param   server: hostname or IP address of RADIUS server
        :type    server: string
        :param authport: port to use for authentication packets
        :type  authport: integer
        :param acctport: port to use for accounting packets
        :type  acctport: integer
        :param   secret: RADIUS secret
        :type    secret: string
        :param     dict: RADIUS dictionary
        :type      dict: pyrad.dictionary.Dictionary
        """
        host.Host.__init__(self, authport, acctport, dict)

        self.server = server
        self.secret = secret
        self._socket = None
        self.retries = 3
        self.timeout = 5

    def bind(self, addr):
        """Bind socket to an address.
        Binding the socket used for communicating to an address can be
        usefull when working on a machine with multiple addresses.

        :param addr: network address (hostname or IP) and port to bind to
        :type  addr: host,port tuple
        """
        self._CloseSocket()
        self._SocketOpen()
        self._socket.bind(addr)

    def _SocketOpen(self):
        if not self._socket:
            self._socket = socket.socket(socket.AF_INET,
                                       socket.SOCK_DGRAM)
            self._socket.setsockopt(socket.SOL_SOCKET,
                                    socket.SO_REUSEADDR, 1)

    def _CloseSocket(self):
        if self._socket:
            self._socket.close()
            self._socket = None

    def CreateAuthPacket(self, **args):
        """Create a new RADIUS packet.
        This utility function creates a new RADIUS packet which can
        be used to communicate with the RADIUS server this client
        talks to. This is initializing the new packet with the
        dictionary and secret used for the client.

        :return: a new empty packet instance
        :rtype:  pyrad.packet.Packet
        """
        return host.Host.CreateAuthPacket(self, secret=self.secret, **args)

    def CreateAcctPacket(self, **args):
        """Create a new RADIUS packet.
        This utility function creates a new RADIUS packet which can
        be used to communicate with the RADIUS server this client
        talks to. This is initializing the new packet with the
        dictionary and secret used for the client.

        :return: a new empty packet instance
        :rtype:  pyrad.packet.Packet
        """
        return host.Host.CreateAcctPacket(self, secret=self.secret, **args)

    def CreateCoAPacket(self, **args):
        """Create a new RADIUS packet.
        This utility function creates a new RADIUS packet which can
        be used to communicate with the RADIUS server this client
        talks to. This is initializing the new packet with the
        dictionary and secret used for the client.
        :return: a new empty packet instance
        :rtype:  pyrad.packet.Packet
        """
        return host.Host.CreateCoAPacket(self, secret=self.secret, **args)        

    def _SendPacket(self, pkt, port):
        """Send a packet to a RADIUS server.

        :param pkt:  the packet to send
        :type pkt:   pyrad.packet.Packet
        :param port: UDP port to send packet to
        :type port:  integer
        :return:     the reply packet received
        :rtype:      pyrad.packet.Packet
        :raise Timeout: RADIUS server does not reply
        """
        self._SocketOpen()

        for attempt in range(self.retries):
            if attempt and pkt.code == packet.AccountingRequest:
                if "Acct-Delay-Time" in pkt:
                    pkt["Acct-Delay-Time"] = \
                            pkt["Acct-Delay-Time"][0] + self.timeout
                else:
                    pkt["Acct-Delay-Time"] = self.timeout
            self._socket.sendto(pkt.RequestPacket(), (self.server, port))

            now = time.time()
            waitto = now + self.timeout

            while now < waitto:
                ready = select.select([self._socket], [], [],
                                    (waitto - now))

                if ready[0]:
                    rawreply = self._socket.recv(4096)
                else:
                    now = time.time()
                    continue

                try:
                    reply = pkt.CreateReply(packet=rawreply)
                    if pkt.VerifyReply(reply, rawreply):
                        return reply
                except packet.PacketError:
                    pass

                now = time.time()

        raise Timeout

    def SendPacket(self, pkt):
        """Send a packet to a RADIUS server.

        :param pkt: the packet to send
        :type pkt:  pyrad.packet.Packet
        :return:    the reply packet received
        :rtype:     pyrad.packet.Packet
        :raise Timeout: RADIUS server does not reply
        """
        if isinstance(pkt, packet.AuthPacket):
            return self._SendPacket(pkt, self.authport)
        else:
            return self._SendPacket(pkt, self.acctport)



#!/usr/bin/env python
#coding=utf-8
from beaker.cache import CacheManager
from sqlalchemy.sql import text as _sql
import functools
import settings
import datetime

__cache_timeout__ = 600

cache = CacheManager(cache_regions={
      'short_term':{ 'type': 'memory', 'expire': __cache_timeout__ }
      }) 

###############################################################################
# Basic Define                                                            ####
###############################################################################

ticket_fds = [
    'account_number','acct_fee','acct_input_gigawords','acct_input_octets',
    'acct_input_packets','acct_output_gigawords','acct_output_octets',
    'acct_output_packets','acct_session_id','acct_session_time',
    'acct_start_time','acct_stop_time','acct_terminate_cause',
    'calling_station_id','fee_receivables','frame_netmask',
    'framed_ipaddr','is_deduct','nas_class','nas_addr',
    'nas_port','nas_port_id','nas_port_type','service_type',
    'session_timeout','start_source','stop_source',"mac_addr"
]


###############################################################################
# Database Store                                                          ####
###############################################################################        

class Store():

    def __init__(self,config,db_engine):
        global __cache_timeout__
        __cache_timeout__ = config.get("radiusd",'cache_timeout')
        self.db_engine = db_engine

    @cache.cache('get_param',expire=__cache_timeout__)   
    def get_param(self,param_name):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select param_value from  slc_param where param_name = :param_name"),param_name=param_name)
            param = cur.fetchone()
            return param and param['param_value'] or None

    def update_param_cache(self):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select param_name from  slc_param "))
            for param in cur:
                cache.invalidate(self.get_param,'get_param', str(param['param_name']))
                cache.invalidate(self.get_param,'get_param', unicode(param['param_name']))

    ###############################################################################
    # bas method                                                              ####
    ############################################################################### 

    def list_bas(self):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select * from  slc_rad_bas"))
            return [bas for bas in cur] 

    @cache.cache('get_bas',expire=__cache_timeout__)   
    def get_bas(self,ipaddr):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select * from slc_rad_bas where ip_addr = :ip_addr"),ip_addr=ipaddr)
            bas = cur.fetchone()
            return bas

    def update_bas_cache(self,ipaddr):
        cache.invalidate(self.get_bas,'get_bas',str(ipaddr))
        cache.invalidate(self.get_bas,'get_bas',unicode(ipaddr))

    ###############################################################################
    # user method                                                              ####
    ###############################################################################  

    @cache.cache('get_user',expire=__cache_timeout__)   
    def get_user(self,username):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select a.*,p.product_policy from slc_rad_account a,slc_rad_product p \
                where a.product_id = p.id and a.account_number = :account "),account=username)
            return  cur.fetchone()


    @cache.cache('get_user_attrs',expire=__cache_timeout__)   
    def get_user_attrs(self,username):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select * from slc_rad_account_attr where account_number = :account "),account=username)
            return cur.fetchall()  

    def get_user_balance(self,username):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select balance from slc_rad_account where account_number = :account "),account=username)
            b = cur.fetchone()  
            return b and b['balance'] or 0    
    
    def get_user_time_length(self,username):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select time_length from slc_rad_account where account_number = :account "),account=username)
            b = cur.fetchone()  
            return b and b['time_length'] or 0
    
    def get_user_flow_length(self,username):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select flow_length from slc_rad_account where account_number = :account "),account=username)
            b = cur.fetchone()  
            return b and b['flow_length'] or 0

    def update_user_cache(self,username):
        cache.invalidate(self.get_user,'get_user', str(username))
        cache.invalidate(self.get_user,'get_user', unicode(username))
        cache.invalidate(self.get_user_attrs,'get_user_attrs', str(username))
        cache.invalidate(self.get_user_attrs,'get_user_attrs', unicode(username))

    def update_user_balance(self,username,balance):
        with self.db_engine.begin() as conn:
            sql = _sql("update slc_rad_account set balance = :balance where account_number = :account")
            conn.execute(sql,balance=balance,account=username)
            self.update_user_cache(username)
            
    def update_user_time_length(self,username,time_length):
        with self.db_engine.begin() as conn:
            sql = _sql("update slc_rad_account set time_length = :time_length where account_number = :account")
            conn.execute(sql,time_length=time_length,account=username)
            self.update_user_cache(username)
    
    def update_user_flow_length(self,username,flow_length):
        with self.db_engine.begin() as conn:
            sql = _sql("update slc_rad_account set flow_length = flow_length where account_number = :account")
            conn.execute(sql,flow_length=flow_length,account=username)
            self.update_user_cache(username)

    def update_user_mac(self,username,mac_addr):
        with self.db_engine.begin() as conn:
            sql =_sql("update slc_rad_account set mac_addr = :mac_addr where account_number = :account")
            conn.execute(sql,mac_addr=mac_addr,account=username)
            self.update_user_cache(username)

    def update_user_vlan_id(self,username,vlan_id):
        with self.db_engine.begin() as conn:
            sql = _sql("update slc_rad_account set vlan_id = :vlan_id where account_number = :account")
            conn.execute(sql,vlan_id=vlan_id,account=username)
            self.update_user_cache(username)

    def update_user_vlan_id2(self,username,vlan_id2):
        with self.db_engine.begin() as conn:
            sql = _sql("update slc_rad_account set vlan_id2 = :vlan_id2 where account_number = :account")
            conn.execute(sql,vlan_id2=vlan_id2,account=username)   
            self.update_user_cache(username)     

    ###############################################################################
    # roster method                                                              ####
    ############################################################################### 

    @cache.cache('get_roster',expire=__cache_timeout__)   
    def get_roster(self,mac_addr):
        if mac_addr:
            mac_addr = mac_addr.upper()
        roster = None
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select * from slc_rad_roster where mac_addr = :mac_addr "),mac_addr=mac_addr)
            roster =  cur.fetchone()
        print roster
        if  roster:
            now = create_time = datetime.datetime.now()
            roster_start = datetime.datetime.strptime(roster['begin_time'],"%Y-%m-%d")
            roster_end = datetime.datetime.strptime(roster['end_time'],"%Y-%m-%d")
            if now < roster_start or now > roster_end:
                return None
            return roster
            
    def is_black_roster(self,mac_addr):
        roster = self.get_roster(mac_addr)
        return roster and roster['roster_type'] == 1 or False
        
    def is_white_roster(self,mac_addr):
        roster = self.get_roster(mac_addr)
        return roster and roster['roster_type'] == 0 or False
    
    def update_roster_cache(self,mac_addr):
        cache.invalidate(self.get_roster,'get_roster', str(mac_addr))
        cache.invalidate(self.get_roster,'get_roster', unicode(mac_addr))

    ###############################################################################
    # product method                                                         ####
    ############################################################################### 

    @cache.cache('get_product',expire=__cache_timeout__)   
    def get_product(self,product_id):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select * from slc_rad_product where id = :id "),id=product_id)
            return cur.fetchone()  

    @cache.cache('get_product_attrs',expire=__cache_timeout__)  
    def get_product_attrs(self,product_id):
        with self.db_engine.begin() as conn:
            cur = conn.execute(_sql("select * from slc_rad_product_attr where product_id = :product_id "),product_id=product_id)
            return cur.fetchall()  

    def update_product_cache(self,product_id):
        cache.invalidate(self.get_product,'get_product',product_id)
        cache.invalidate(self.get_product_attrs,'get_product_attrs',product_id)
        
    ###############################################################################
    # cache method                                                            ####
    ###############################################################################
    
    def update_all_cache(self):
        from beaker.cache import cache_managers
        for _cache in cache_managers.values():
            _cache.clear()


    ###############################################################################
    # online method                                                            ####
    ############################################################################### 
          
    def is_online(self,nas_addr,acct_session_id):
        with self.db_engine.begin() as conn:
            sql = _sql('select count(id) as online from slc_rad_online where  nas_addr = :nas_addr and acct_session_id = :session_id')
            cur = conn.execute(sql,nas_addr=nas_addr,session_id=acct_session_id)
            return cur.fetchone()['online'] > 0

    def count_online(self,account_number):
        with self.db_engine.begin() as conn:
            sql = _sql('select count(id) as online from slc_rad_online where  account_number = :account')
            cur = conn.execute(sql,account=account_number) 
            return cur.fetchone()['online']

    def get_online(self,nas_addr,acct_session_id):
        with self.db_engine.begin() as conn:
            sql = _sql('select * from slc_rad_online where  nas_addr = :nas_addr and acct_session_id = :session_id')
            cur = conn.execute(sql,nas_addr=nas_addr,session_id=acct_session_id)
            return cur.fetchone()     

    def get_nas_onlines(self,nas_addr):
        with self.db_engine.begin() as conn:
            sql = _sql('select * from slc_rad_online where nas_addr = :nas_addr')
            cur = conn.execute(sql,nas_addr=nas_addr) 
            return cur.fetchall()        

    def add_online(self,online):
        with self.db_engine.begin() as conn:
            keys = ','.join(online.keys())
            vals = ",".join(["'%s'"% c for c in online.values()])
            sql = _sql('insert into slc_rad_online (%s) values(%s)'%(keys,vals))
            conn.execute(sql)
            
    
    def check_online_over(self):
        onlines = []
        with self.db_engine.begin() as conn:
            sql = _sql('select acct_start_time,nas_addr,acct_session_id from slc_rad_online')
            onlines = conn.execute(sql)
        
        for online in onlines:
            start_time = datetime.datetime.strptime(online['acct_start_time'],"%Y-%m-%d %H:%M:%S")
            _datetime = datetime.datetime.now() 
            if (_datetime - start_time).seconds > 3600 * 3:
                self.unlock_online(
                    online['nas_addr'],
                    online['acct_session_id'],
                    settings.STATUS_TYPE_CHECK_ONLINE
                )
            
              
            
    def update_online(self,online):
        with self.db_engine.begin() as conn:
            online_sql = _sql("""update slc_rad_online set 
                billing_times = :billing_times,
                input_total = :input_total,
                output_total = :output_total
                where nas_addr = :nas_addr and acct_session_id = :session_id
            """)
            conn.execute(online_sql,
                billing_times = online['billing_times'],
                input_total = online['input_total'],
                output_total = online['output_total'],
                nas_addr = online['nas_addr'],
                session_id = online['acct_session_id']
            )
            

    def update_billing(self,billing,time_length=0,flow_length=0):
        with self.db_engine.begin() as conn:
            # update account
            balan_sql = _sql("""update slc_rad_account set 
                balance = :balance,
                time_length=:time_length,
                flow_length=:flow_length
                where account_number = :account
            """)
            conn.execute(balan_sql,
                balance=billing.balance,
                time_length=time_length,
                flow_length=flow_length,
                account=billing.account_number
            )
            
            # update online
            online_sql = _sql("""update slc_rad_online set 
                billing_times = :billing_times,
                input_total = :input_total,
                output_total = :output_total
                where nas_addr = :nas_addr and acct_session_id = :session_id
            """)
            conn.execute(online_sql,
                billing_times=billing.acct_session_time,
                input_total=billing.input_total,
                output_total=billing.output_total,
                nas_addr=billing.nas_addr,
                session_id=billing.acct_session_id
            )
            
            # update billing
            keys = ','.join(billing.keys())
            vals = ",".join(["'%s'"% c for c in billing.values()])
            billing_sql = _sql('insert into slc_rad_billing (%s) values(%s)'%(keys,vals))
            conn.execute(billing_sql)
            
            
        self.update_user_cache(billing.account_number) 
    
    def del_online(self,nas_addr,acct_session_id):
        with self.db_engine.begin() as conn:
            sql = _sql('delete from slc_rad_online where nas_addr = :nas_addr and acct_session_id = :session_id')
            conn.execute(sql,nas_addr=nas_addr,session_id=acct_session_id)
            

    def add_ticket(self,ticket):
        _ticket = ticket.copy()
        for _key in _ticket:
            if _key not in ticket_fds:
                del ticket[_key]
        with self.db_engine.begin() as conn:
            keys = ','.join(ticket.keys())
            vals = ",".join(["'%s'"% c for c in ticket.values()])
            sql = _sql('insert into slc_rad_ticket (%s) values(%s)'%(keys,vals))
            conn.execute(sql)
            

    def unlock_online(self,nas_addr,acct_session_id,stop_source):
        bsql = _sql(''' insert into slc_rad_ticket 
                    (
                        account_number,
                        acct_session_id,
                        acct_start_time,
                        nas_addr,
                        framed_ipaddr,
                        start_source,
                        acct_session_time,
                        acct_stop_time,
                        stop_source
                    ) values(
                        :account_number,
                        :acct_session_id,
                        :acct_start_time,
                        :nas_addr,
                        :framed_ipaddr,
                        :start_source,
                        :acct_session_time,
                        :acct_stop_time,
                        :stop_source
                    )''') 
                    
        def _ticket(online):
            _datetime = datetime.datetime.now()
            _starttime = datetime.datetime.strptime(online['acct_start_time'],"%Y-%m-%d %H:%M:%S")
            session_time = (_datetime - _starttime).seconds
            stop_time = _datetime.strftime( "%Y-%m-%d %H:%M:%S")
            return dict(
                account_number = online['account_number'],
                acct_session_id = online['acct_session_id'],
                acct_start_time = online['acct_start_time'],
                nas_addr = online['nas_addr'],
                framed_ipaddr = online['framed_ipaddr'],
                start_source = online['start_source'], 
                acct_session_time = session_time,
                acct_stop_time = stop_time,
                stop_source = stop_source
            )

        def _unlock_one():
            ticket = None
            with self.db_engine.begin() as conn:
                sql = _sql('select * from slc_rad_online where  nas_addr = :nas_addr and acct_session_id = :session_id')
                cur = conn.execute(sql,nas_addr=nas_addr,session_id=acct_session_id)
                online = cur.fetchone()
                if online:
                    ticket = _ticket(online) 
                    dsql = _sql('delete from slc_rad_online where nas_addr = :nas_addr and acct_session_id = :session_id')
                    conn.execute(dsql,nas_addr=nas_addr,session_id=acct_session_id)
                    conn.execute(bsql,**ticket)
                      

        def _unlock_many():
            tickets = []
            with self.db_engine.begin() as conn:
                cur = conn.execute(_sql('select * from slc_rad_online where nas_addr = :nas_addr'),nas_addr=nas_addr) 
                for online in cur:  
                    conn.execute(bsql,**_ticket(online))
                    conn.execute(_sql('delete from slc_rad_online where nas_addr = :nas_addr'),nas_addr=nas_addr)
                                      

        if acct_session_id:_unlock_one()
        else:_unlock_many()

    




#!/usr/bin/env python
#coding=utf-8
from toughradius.radiusd.plugins import error_auth

def process(req=None,resp=None,user=None,radiusd=None,**kwargs):
    """check block roster"""
    store = radiusd.store
    macaddr = req.get_mac_addr()
    if store.is_black_roster(macaddr):
        return error_auth(resp,"user macaddr in blacklist")
    return resp


#!/usr/bin/env python
#coding:utf-8
import os
from toughradius.tools.dbengine import get_engine
from sqlalchemy.sql import text as _sql
from toughradius.tools.shell import shell as sh

def _decode(src):
    try:
        return unicode(src,'utf-8')
    except:
        return unicode(src)

def print_result(rs):
    n = 0
    rstr = []
    for r in rs:
        if n == 100:
            return
        try:
            _r = ', '.join([ '%s=%s'%(k,_decode(r[k])) for k in r.keys()])
            rstr.append(_r)
        except Exception as err:
            sh.warn('print result error: %s'%str(err))
    sh.info('Result:\n\n%s\n'%('\n'.join(rstr)))        
    
            
        

def execute_sqls(config,sqlstr):
    sh.info('exec sql >> %s'%sqlstr)
    results = []
    with get_engine(config).begin() as conn:
        try:
            results = conn.execute(_sql(sqlstr))
        except Exception as err:
            return sh.err('exec sql error: %s'%str(err))
    sh.info('exec sql done')
    print_result(results)
    

def execute_sqlf(config,sqlfile):
    sh.info('exec sql file >> %s'%sqlfile)
    if not os.path.exists(sqlfile):
        return sh.warn('sqlfile not exists')
    conn = get_engine(config).raw_connection()
    try:
        for line in open(sqlfile):
            sh.info(line)
            conn.execute(line)
    except Exception as err:
        return sh.err('exec sql file error: %s'%str(err))
    finally:
        conn.close()
    sh.info('exec sql file done')

    
    
    


# host.py
#
# Copyright 2003,2007 Wichert Akkerman <wichert@wiggy.net>

import packet


class Host:
    """Generic RADIUS capable host.

    :ivar     dict: RADIUS dictionary
    :type     dict: pyrad.dictionary.Dictionary
    :ivar authport: port to listen on for authentication packets
    :type authport: integer
    :ivar acctport: port to listen on for accounting packets
    :type acctport: integer
    """
    def __init__(self, authport=1812, acctport=1813, dict=None):
        """Constructor

        :param authport: port to listen on for authentication packets
        :type  authport: integer
        :param acctport: port to listen on for accounting packets
        :type  acctport: integer
        :param     dict: RADIUS dictionary
        :type      dict: pyrad.dictionary.Dictionary
        """
        self.dict = dict
        self.authport = authport
        self.acctport = acctport

    def CreatePacket(self, **args):
        """Create a new RADIUS packet.
        This utility function creates a new RADIUS authentication
        packet which can be used to communicate with the RADIUS server
        this client talks to. This is initializing the new packet with
        the dictionary and secret used for the client.

        :return: a new empty packet instance
        :rtype:  pyrad.packet.Packet
        """
        return packet.Packet(dict=self.dict, **args)

    def CreateAuthPacket(self, **args):
        """Create a new authentication RADIUS packet.
        This utility function creates a new RADIUS authentication
        packet which can be used to communicate with the RADIUS server
        this client talks to. This is initializing the new packet with
        the dictionary and secret used for the client.

        :return: a new empty packet instance
        :rtype:  pyrad.packet.AuthPacket
        """
        return packet.AuthPacket(dict=self.dict, **args)

    def CreateAcctPacket(self, **args):
        """Create a new accounting RADIUS packet.
        This utility function creates a new accouting RADIUS packet
        which can be used to communicate with the RADIUS server this
        client talks to. This is initializing the new packet with the
        dictionary and secret used for the client.

        :return: a new empty packet instance
        :rtype:  pyrad.packet.AcctPacket
        """
        return packet.AcctPacket(dict=self.dict, **args)

    def CreateCoAPacket(self, **args):
        """Create a new RADIUS packet.
        This utility function creates a new RADIUS packet which can
        be used to communicate with the RADIUS server this client
        talks to. This is initializing the new packet with the
        dictionary and secret used for the client.
        :return: a new empty packet instance
        :rtype:  pyrad.packet.Packet
        """
        return host.Host.CreateCoAPacket(self, secret=self.secret, **args)        

    def SendPacket(self, fd, pkt):
        """Send a packet.

        :param fd: socket to send packet with
        :type  fd: socket class instance
        :param pkt: packet to send
        :type  pkt: Packet class instance
        """
        fd.sendto(pkt.Packet(), pkt.source)

    def SendReplyPacket(self, fd, pkt):
        """Send a packet.

        :param fd: socket to send packet with
        :type  fd: socket class instance
        :param pkt: packet to send
        :type  pkt: Packet class instance
        """
        fd.sendto(pkt.ReplyPacket(), pkt.source)



#!/usr/bin/env python
#coding=utf-8
from twisted.python import log
from toughradius.radiusd.settings import *
import logging
import json

def process(req=None,admin=None,**kwargs):
    msg_id = req.get("msg_id")
    nas_addr = req.get("nas_addr") 
    if not nas_addr:
        reply = json.dumps({'msg_id':msg_id,'data':u'nas_addr is empty','code':1})
        return admin.sendMessage(reply,False) 
    session_id = req.get("acct_session_id")
    admin.radiusd.store.unlock_online(nas_addr,session_id,STATUS_TYPE_UNLOCK)
    reply = json.dumps({'msg_id':msg_id,'data':u'unlock ok','code':0})
    admin.sendMessage(reply,False)



#!/usr/bin/env python
#coding:utf-8

from bottle import Bottle
from bottle import request
from bottle import response
from bottle import redirect
from bottle import static_file
from tablib import Dataset
from toughradius.console.websock import websock
from toughradius.console import models
from toughradius.console.libs import utils
from toughradius.console.admin import forms
from toughradius.console.base import *
import bottle
import datetime
from sqlalchemy import func

__prefix__ = "/card"

app = Bottle()
app.config['__prefix__'] = __prefix__
render = functools.partial(Render.render_app,app)


@app.get('/calc',apply=auth_opr)
def card_calc(db):
    product_id = request.params.get('product_id')
    product = db.query(models.SlcRadProduct).get(product_id)
    #预付费包月
    if product and product.product_policy == 0:
        return dict(code=0,data={
            'fee_value' : utils.fen2yuan(product.fee_price),
            'months' : 1
        })
    # 买断包月 买断时长 买断流量
    elif product and product.product_policy in (2,3,5):
        return dict(code=0,data={
            'fee_value' : utils.fen2yuan(product.fee_price),
            'months' : product.fee_months,
            'times' : utils.sec2hour(product.fee_times),
            'flows' : utils.kb2mb(product.fee_flows)
        })
    else:
        return dict(code=1,data=u"不支持的资费")
    
@app.route('/list',apply=auth_opr,method=['GET','POST'])
@app.post('/export',apply=auth_opr)
def card_list(db):   
    product_id = request.params.get('product_id')
    card_type = request.params.get('card_type') 
    card_status = request.params.get('card_status')
    batch_no = request.params.get('batch_no')
    query_begin_time = request.params.get('query_begin_time')
    query_end_time = request.params.get('query_end_time')
    
    _query = db.query(models.SlcRechargerCard)
    if product_id and card_type == '0':
        _query = _query.filter(models.SlcRechargerCard.product_id==product_id)
    if card_type:
        _query = _query.filter(models.SlcRechargerCard.card_type==card_type)
    if batch_no:
        _query = _query.filter(models.SlcRechargerCard.batch_no==batch_no)
    if card_status:
        _query = _query.filter(models.SlcRechargerCard.card_status==card_status)
    if query_begin_time:
        _query = _query.filter(models.SlcRechargerCard.create_time >= query_begin_time+' 00:00:00')
    if query_end_time:
        _query = _query.filter(models.SlcRechargerCard.create_time <= query_end_time+' 23:59:59')
    
    products = db.query(models.SlcRadProduct).filter(
        models.SlcRadProduct.product_status == 0,
        models.SlcRadProduct.product_policy.in_([0,2,3,5])
    )
    if request.path == '/list':
        print "total:",_query.count()
        return render("card_list", 
            page_data = get_page_data(_query),
            card_types = forms.card_types,
            card_states = forms.card_states,
            products = products,
            colors = {0:'',1:'class="success"',2:'class="warning"',3:'class="danger"'},
            **request.params
        )
    elif request.path == '/export':
        data = Dataset()
        data.append((
            u'批次号',u'充值卡号',u'充值卡密码',u'充值卡类型',u'状态',
            u'资费id', u'面值/售价',u"授权月数",u"授权时长(小时)",u"授权流量(MB)",u"过期时间",u'创建时间'
         ))
        print "total:",_query.count()
        for i in _query:
            data.append((
                i.batch_no, i.card_number, utils.decrypt(i.card_passwd),forms.card_types[i.card_type],
                forms.card_states[i.card_status],get_product_name(db,i.product_id),utils.fen2yuan(i.fee_value),
                i.months,utils.sec2hour(i.times),utils.kb2mb(i.flows),i.expire_date,i.create_time
            ))
        name = u"RADIUS-CARD-" + datetime.datetime.now().strftime("%Y%m%d-%H%M%S") + ".xls"
        return export_file(name,data)
        
    
permit.add_route("%s/list"%__prefix__,u"充值卡管理",u"系统管理",is_menu=True,order=7)
permit.add_route("%s/export"%__prefix__,u"充值卡导出",u"系统管理",order=7.01)

@app.get('/create',apply=auth_opr)
def card_create(db):
    products = [ (n.id,n.product_name) for n in db.query(models.SlcRadProduct).filter(
        models.SlcRadProduct.product_status == 0,
        models.SlcRadProduct.product_policy.in_([0,2,3,5])
    )]
    batch_no = datetime.datetime.now().strftime("%Y%m%d")
    form = forms.recharge_card_form(products)
    form.batch_no.set_value(batch_no)
    return render("card_form",form=form)
    
@app.post('/create',apply=auth_opr)
def card_create(db):
    def gencardpwd(clen=8):
        r = list('1234567890abcdefghijklmnopqrstuvwxyz')
        rg = utils.random_generator
        return utils.encrypt(''.join([rg.choice(r) for _ in range(clen)]))
        
    products = [ (n.id,n.product_name) for n in db.query(models.SlcRadProduct).filter(
        models.SlcRadProduct.product_status == 0,
        models.SlcRadProduct.product_policy.in_([0,2,3,5])
    )]
    form = forms.recharge_card_form(products)
    if not form.validates(source=request.forms):
        return render("card_form",form=form)
    card_type = int(form.d.card_type)
    batch_no = form.d.batch_no
    if len(batch_no) != 8:
        return render("card_form",form=form,msg=u"批次号必须是8位数字")
    
    pwd_len = int(form.d.pwd_len)
    if pwd_len > 16:
        pwd_len = 16

    start_card = int(form.d.start_no)
    stop_card = int(form.d.stop_no)
    if start_card > stop_card:
        return render("card_form",form=form,msg=u"开始卡号不能大于结束卡号")
    
    if form.d.expire_date < utils.get_currdate():
        return render("card_form",form=form,msg=u"过期时间不能小于今天")
    
    fee_value = utils.yuan2fen(form.d.fee_value)
    if fee_value == 0 and card_type == 1:
        return render("card_form",form=form,msg=u"不能发行余额为0的余额卡")
    
    for _card in range(start_card,stop_card+1):
        card_number = "%s%s"%(batch_no,str(_card).zfill(5))
        card_obj = models.SlcRechargerCard()
        card_obj.batch_no = batch_no
        card_obj.card_number = card_number
        card_obj.card_passwd = gencardpwd(pwd_len)
        card_obj.card_type = card_type
        card_obj.card_status = 0
        card_obj.product_id = card_type==0 and form.d.product_id or -1
        card_obj.fee_value = fee_value
        card_obj.months = card_type==0 and int(form.d.months) or 0
        card_obj.times = card_type==0 and utils.hour2sec(form.d.times) or 0
        card_obj.flows = card_type==0 and utils.mb2kb(form.d.flows) or 0
        card_obj.expire_date = form.d.expire_date
        card_obj.create_time = utils.get_currtime()
        db.add(card_obj)
    
    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)生成批次[%s]的[%s]'%(get_cookie("username"),batch_no,forms.card_types[card_type])
    db.add(ops_log)
    db.commit()
    path = "%s/list?card_type=%s&query_begin_time=%s"%(__prefix__,card_type,utils.get_currdate())
    if form.d.product_id:
        path = "%s&product_id=%s"%(path,form.d.product_id)
    redirect(path)

permit.add_route("%s/create"%__prefix__,u"充值卡生成",u"系统管理",order=7.02)

@app.get('/active',apply=auth_opr)
def card_active(db):
    card_id = request.params.get("card_id")
    if not card_id:
        return dict(code=0,msg=u"非法的访问")
    card = db.query(models.SlcRechargerCard).get(card_id)
    if not card:
        return dict(code=0,msg=u"充值卡不存在")
    card.card_status = 1
    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)激活充值卡[%s]'%(get_cookie("username"),card.card_number)
    db.add(ops_log)
    db.commit()
    return dict(code=0,msg=u"激活成功，充值卡已可使用")
    

permit.add_route("%s/active"%__prefix__,u"充值卡激活",u"系统管理",order=7.03)


@app.get('/recycle',apply=auth_opr)
def card_recycle(db):
    card_id = request.params.get("card_id")
    if not card_id:
        return dict(code=0,msg=u"非法的访问")
    card = db.query(models.SlcRechargerCard).get(card_id)
    if not card:
        return dict(code=0,msg=u"充值卡不存在")
    card.card_status = 3
    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'操作员(%s)回收充值卡[%s]'%(get_cookie("username"),card.card_number)
    db.add(ops_log)
    db.commit()
    return dict(code=0,msg=u"回收成功，充值卡已不可使用")
    

permit.add_route("%s/recycle"%__prefix__,u"充值卡回收",u"系统管理",order=7.04)


#!/usr/bin/env python
#coding:utf-8
import functools
from sqlalchemy import create_engine
ISOLATION_LEVEL = {
    1 : 'READ COMMITTED',
    2 : 'READ UNCOMMITTED',
    3 : 'REPEATABLE READ',
    4 : 'SERIALIZABLE'
}


class DBEngine(object):

    def __init__(self,config):
        self.config = config
        self.conf_str = functools.partial(self.config.get,"database")
        self.conf_int = functools.partial(self.config.getint,"database")
        self.conf_bool = functools.partial(self.config.getboolean,"database")
        
    def __call__(self):
        return self.get_engine()

    def get_engine(self):
        if self.conf_str('dbtype') == 'mysql':
            return create_engine(
                self.conf_str("dburl"),
                echo=self.conf_bool('echo'),
                pool_size = self.conf_int('pool_size'),
                pool_recycle=self.conf_int('pool_recycle')
            )
        elif self.conf_str('dbtype') == 'postgresql':
            return create_engine(
                self.conf_str("dburl"),
                echo=self.conf_bool('echo'),
                pool_size = self.conf_int('pool_size'),
                isolation_level = ISOLATION_LEVEL.get(self.conf_int('isolation_level'),1),
                pool_recycle=self.conf_int('pool_recycle')
            )
        elif self.conf_str('dbtype') == 'sqlite':
            def my_con_func():
                import sqlite3.dbapi2 as sqlite
                con = sqlite.connect(self.conf_str("dburl").replace('sqlite:///',''))
                con.text_factory=str
                con.execute("PRAGMA synchronous=OFF;")
                con.isolation_level = 'IMMEDIATE'
                return con
            return create_engine(
                "sqlite+pysqlite:///",
                creator=my_con_func,
                echo=self.conf_bool('echo')
            )
        else:
            return create_engine(
                self.conf_str("dburl"),
                echo=self.conf_bool('echo'),
                pool_size = self.conf_int('pool_size')
            )

def get_engine(config):
    return DBEngine(config)()


#!/usr/bin/env python
#coding:utf-8
__all__ = [
  "validipaddr", "validipport", "validip", "validaddr", 
  "urlquote",
  "httpdate", "parsehttpdate", 
  "htmlquote", "htmlunquote", "websafe",
]

import urllib, time
try: import datetime
except ImportError: pass

def validipaddr(address):
    """
    Returns True if `address` is a valid IPv4 address.
    
        >>> validipaddr('192.168.1.1')
        True
        >>> validipaddr('192.168.1.800')
        False
        >>> validipaddr('192.168.1')
        False
    """
    try:
        octets = address.split('.')
        if len(octets) != 4:
            return False
        for x in octets:
            if not (0 <= int(x) <= 255):
                return False
    except ValueError:
        return False
    return True

def validipport(port):
    """
    Returns True if `port` is a valid IPv4 port.
    
        >>> validipport('9000')
        True
        >>> validipport('foo')
        False
        >>> validipport('1000000')
        False
    """
    try:
        if not (0 <= int(port) <= 65535):
            return False
    except ValueError:
        return False
    return True

def validip(ip, defaultaddr="0.0.0.0", defaultport=8080):
    """Returns `(ip_address, port)` from string `ip_addr_port`"""
    addr = defaultaddr
    port = defaultport
    
    ip = ip.split(":", 1)
    if len(ip) == 1:
        if not ip[0]:
            pass
        elif validipaddr(ip[0]):
            addr = ip[0]
        elif validipport(ip[0]):
            port = int(ip[0])
        else:
            raise ValueError, ':'.join(ip) + ' is not a valid IP address/port'
    elif len(ip) == 2:
        addr, port = ip
        if not validipaddr(addr) and validipport(port):
            raise ValueError, ':'.join(ip) + ' is not a valid IP address/port'
        port = int(port)
    else:
        raise ValueError, ':'.join(ip) + ' is not a valid IP address/port'
    return (addr, port)

def validaddr(string_):
    """
    Returns either (ip_address, port) or "/path/to/socket" from string_
    
        >>> validaddr('/path/to/socket')
        '/path/to/socket'
        >>> validaddr('8000')
        ('0.0.0.0', 8000)
        >>> validaddr('127.0.0.1')
        ('127.0.0.1', 8080)
        >>> validaddr('127.0.0.1:8000')
        ('127.0.0.1', 8000)
        >>> validaddr('fff')
        Traceback (most recent call last):
            ...
        ValueError: fff is not a valid IP address/port
    """
    if '/' in string_:
        return string_
    else:
        return validip(string_)

def urlquote(val):
    """
    Quotes a string for use in a URL.
    
        >>> urlquote('://?f=1&j=1')
        '%3A//%3Ff%3D1%26j%3D1'
        >>> urlquote(None)
        ''
        >>> urlquote(u'\u203d')
        '%E2%80%BD'
    """
    if val is None: return ''
    if not isinstance(val, unicode): val = str(val)
    else: val = val.encode('utf-8')
    return urllib.quote(val)

def httpdate(date_obj):
    """
    Formats a datetime object for use in HTTP headers.
    
        >>> import datetime
        >>> httpdate(datetime.datetime(1970, 1, 1, 1, 1, 1))
        'Thu, 01 Jan 1970 01:01:01 GMT'
    """
    return date_obj.strftime("%a, %d %b %Y %H:%M:%S GMT")

def parsehttpdate(string_):
    """
    Parses an HTTP date into a datetime object.

        >>> parsehttpdate('Thu, 01 Jan 1970 01:01:01 GMT')
        datetime.datetime(1970, 1, 1, 1, 1, 1)
    """
    try:
        t = time.strptime(string_, "%a, %d %b %Y %H:%M:%S %Z")
    except ValueError:
        return None
    return datetime.datetime(*t[:6])

def htmlquote(text):
    r"""
    Encodes `text` for raw use in HTML.
    
        >>> htmlquote(u"<'&\">")
        u'&lt;&#39;&amp;&quot;&gt;'
    """
    text = text.replace(u"&", u"&amp;") # Must be done first!
    text = text.replace(u"<", u"&lt;")
    text = text.replace(u">", u"&gt;")
    text = text.replace(u"'", u"&#39;")
    text = text.replace(u'"', u"&quot;")
    return text

def htmlunquote(text):
    r"""
    Decodes `text` that's HTML quoted.

        >>> htmlunquote(u'&lt;&#39;&amp;&quot;&gt;')
        u'<\'&">'
    """
    text = text.replace(u"&quot;", u'"')
    text = text.replace(u"&#39;", u"'")
    text = text.replace(u"&gt;", u">")
    text = text.replace(u"&lt;", u"<")
    text = text.replace(u"&amp;", u"&") # Must be done last!
    return text
    
def websafe(val):
    r"""Converts `val` so that it is safe for use in Unicode HTML.

        >>> websafe("<'&\">")
        u'&lt;&#39;&amp;&quot;&gt;'
        >>> websafe(None)
        u''
        >>> websafe(u'\u203d')
        u'\u203d'
        >>> websafe('\xe2\x80\xbd')
        u'\u203d'
    """
    if val is None:
        return u''
    elif isinstance(val, str):
        val = val.decode('utf-8')
    elif not isinstance(val, unicode):
        val = unicode(val)
        
    return htmlquote(val)

if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python
from twisted.python import log
from twisted.internet import reactor
from twisted.internet import threads
from twisted.web.client import getPage
from twisted.internet import task
from sqlalchemy.sql import func
from toughradius.console import models
from toughradius.console.libs.smail import mail
from urllib import quote
import datetime
import time

def __online_stat_job(mk_db):
    def execute(mk_db):
        log.msg("start online stat task..")
        db = mk_db()
        try:
            nodes = db.query(models.SlcNode)
            for node in nodes:
                online_count = db.query(models.SlcRadOnline.id).filter(
                    models.SlcRadOnline.account_number == models.SlcRadAccount.account_number,
                    models.SlcRadAccount.member_id == models.SlcMember.member_id,
                    models.SlcMember.node_id == node.id
                ).count()
                stat = models.SlcRadOnlineStat()
                stat.node_id = node.id
                stat.stat_time = int(time.time())
                stat.total = online_count
                db.add(stat)
            db.commit()
            log.msg("online stat task done")
        except Exception as err:
            db.rollback()
            log.err(err,'online_stat_job err')
        finally:
            db.close()

    reactor.callInThread(execute,mk_db)

def __flow_stat_job(mk_db):
    def execute(mk_db):
        log.msg("start flow stat task..")
        db = mk_db()
        try:
            nodes = db.query(models.SlcNode)
            for node in nodes:
                r = db.query(
                    func.sum(models.SlcRadOnline.input_total).label("input_total"),
                    func.sum(models.SlcRadOnline.output_total).label("output_total")
                ).filter(
                    models.SlcRadOnline.account_number == models.SlcRadAccount.account_number,
                    models.SlcRadAccount.member_id == models.SlcMember.member_id,
                    models.SlcMember.node_id == node.id
                ).first()
                if r:
                    stat = models.SlcRadFlowStat()
                    stat.node_id = node.id
                    stat.stat_time = int(time.time())
                    stat.input_total = r.input_total
                    stat.output_total = r.output_total
                    db.add(stat)
            db.commit()
            log.msg("flow stat task done")
        except  Exception as err:
            db.rollback()
            log.err(err,'flow_stat_job err')
        finally:
            db.close()

    reactor.callInThread(execute,mk_db)


def __expire_notify(mk_db):
    def _getpage(url):
        errback = lambda x:log.err(x,'getPage error')
        d = getPage(url)
        d.addErrback(errback)
        
    log.msg("start expire notify task..")
    db = mk_db()
    try:
        _ndays = db.query(models.SlcParam.param_value).filter_by(
            param_name = 'expire_notify_days').scalar()

        notify_tpl = db.query(models.SlcParam.param_value).filter_by(
            param_name = 'expire_notify_tpl').scalar()

        notify_url = db.query(models.SlcParam.param_value).filter_by(
            param_name = 'expire_notify_url').scalar()

        _now = datetime.datetime.now()
        _date = (datetime.datetime.now() + datetime.timedelta(days=int(_ndays))).strftime("%Y-%m-%d")
        expire_query = db.query(
            models.SlcRadAccount.account_number,
            models.SlcRadAccount.expire_date,
            models.SlcMember.email,
            models.SlcMember.mobile
        ).filter(
            models.SlcRadAccount.member_id == models.SlcMember.member_id,
            models.SlcRadAccount.expire_date <= _date,
            models.SlcRadAccount.expire_date >= _now.strftime("%Y-%m-%d"),
            models.SlcRadAccount.status == 1
        )

        log.msg('expire_notify total: %s'%expire_query.count())
        commands = []
        for account,expire,email,mobile in expire_query:
            ctx = notify_tpl.replace('#account#',account)
            ctx = ctx.replace('#expire#',expire)
            topic = ctx[:ctx.find('\n')]
            commands.append( (mail.sendmail,[email,topic,ctx],{}) )
            
            url = notify_url.replace('{account}',account)
            url = url.replace('{expire}',expire)
            url = url.replace('{email}',email)
            url = url.replace('{mobile}',mobile)
            url = url.encode('utf-8')
            url = quote(url,":?=/&")
            commands.append( (_getpage,[url],{}) )

        threads.callMultipleInThread(commands)
        log.msg("expire_notify to ansync task")

    except Exception as err:
        db.rollback()
        log.err(err,'expire notify erro')
    finally:
        db.close()

def start_online_stat_job(mk_db):
    print ('start online_stat_job...')
    _task = task.LoopingCall(__online_stat_job,mk_db)
    _task.start(300)

def start_flow_stat_job(mk_db):
    print ('start flow_stat_job...')
    _task = task.LoopingCall(__flow_stat_job,mk_db)
    _task.start(300)

def start_expire_notify_job(mk_db):
    print ('start flow_stat_job...')
    _task = task.LoopingCall(__expire_notify,mk_db)
    _task.start(3600*12)



#!/usr/bin/env python
#coding=utf-8
from twisted.python import log
from toughradius.radiusd.pyrad import packet
from toughradius.radiusd.settings import *
from toughradius.radiusd import utils
import logging
import datetime

def process(req=None,user=None,radiusd=None,**kwargs):
    if not req.get_acct_status_type() == STATUS_TYPE_UPDATE:
        return   

    if not user:
        return log.err("[Acct] Received an accounting update request but user[%s] not exists"%req.get_user_name())      

    runstat=radiusd.runstat
    store = radiusd.store
    
    runstat.acct_update += 1  
    online = store.get_online(req.get_nas_addr(),req.get_acct_sessionid())  

    if not online:         
        sessiontime = req.get_acct_sessiontime()
        updatetime = datetime.datetime.now()
        _starttime = updatetime - datetime.timedelta(seconds=sessiontime)       
        online = utils.Storage(
            account_number = user['account_number'],
            nas_addr = req.get_nas_addr(),
            acct_session_id = req.get_acct_sessionid(),
            acct_start_time = _starttime.strftime( "%Y-%m-%d %H:%M:%S"),
            framed_ipaddr = req.get_framed_ipaddr(),
            mac_addr = req.get_mac_addr(),
            nas_port_id = req.get_nas_portid(),
            billing_times = req.get_acct_sessiontime(),
            input_total = req.get_input_total(),
            output_total = req.get_output_total(),
            start_source = STATUS_TYPE_UPDATE
        )
        store.add_online(online)   

    log.msg('%s Accounting update request, update online'%req.get_user_name(),level=logging.INFO)
        


#!/usr/bin/env python
#coding=utf-8
import json

def process(req=None,admin=None,**kwargs):
    data = admin.radiusd.runstat.copy()
    data['msg_id'] = req.get('msg_id')
    admin.sendMessage(json.dumps(data),False) 





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys,os
sys.path.insert(0,os.path.split(__file__)[0])
sys.path.insert(0,os.path.abspath(os.path.pardir))
from toughradius.console import models
from toughradius.console.libs import utils
import argparse,ConfigParser

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-c','--conf', type=str,default='../etc/radiusd.conf',dest='conf',help='conf file')
    parser.add_argument('-u','--update',action='store_true',default=False,dest='update',help='update option')
    parser.add_argument('-i','--install',action='store_true',default=False,dest='install',help='install option')
    args =  parser.parse_args(sys.argv[1:])    
    # read config file
    config = ConfigParser.ConfigParser()
    config.read(args.conf)
    utils.aescipher.setup(config.get('DEFAULT','secret'))
    
    if args.update:
        models.update(config)
    elif args.install:
        models.install2(config)
    else:
        models.install(config)

        


#coding:utf-8
from toughradius.console.libs import pyforms
from toughradius.console.libs.pyforms import dataform
from toughradius.console.libs.pyforms import rules
from toughradius.console.libs.pyforms.rules import button_style,input_style

boolean = {0:u"否", 1:u"是"}

param_form = pyforms.Form(
    pyforms.Textbox("system_name", description=u"管理系统名称", **input_style),
    pyforms.Textbox("customer_system_name", description=u"自助服务系统名称", **input_style),
    pyforms.Textbox("customer_system_url", description=u"自助服务系统网站地址", **input_style),
    pyforms.Textbox("customer_must_active", description=u"激活邮箱才能自助开户充值(0:否|1:是)",hr=True, **input_style),
    pyforms.Textbox("weixin_qrcode", description=u"微信公众号二维码图片(宽度230px)", **input_style),
    pyforms.Textbox("service_phone", description=u"客户服务电话", **input_style),
    pyforms.Textbox("service_qq", description=u"客户服务QQ号码", **input_style),
    pyforms.Textbox("rcard_order_url", description=u"充值卡订购网站地址",hr=True,**input_style),
    pyforms.Textbox("expire_notify_days",rules.is_number, description=u"到期提醒提前天数", **input_style),
    pyforms.Textarea("expire_notify_tpl", description=u"到期提醒邮件模板",rows=3, **input_style),
    pyforms.Textbox("expire_notify_url", description=u"到期通知触发URL", **input_style),
    pyforms.Textbox("expire_session_timeout", description=u"到期用户下发最大会话时长(秒)", **input_style),
    pyforms.Textbox("expire_addrpool", description=u"到期提醒下发地址池", hr=True,**input_style),    
    pyforms.Textbox("smtp_server", description=u"SMTP服务器", **input_style),
    pyforms.Textbox("smtp_user", description=u"SMTP用户名", **input_style),
    pyforms.Textbox("smtp_pwd", description=u"SMTP密码",hr=True, **input_style),
    # pyforms.Textbox("smtp_sender", description=u"smtp发送人名称", **input_style),
    pyforms.Textbox("is_debug", description=u"DEBUG模式(0|1)",**input_style),
    pyforms.Textbox("radiusd_address", description=u"Radius服务IP地址",**input_style),
    pyforms.Textbox("radiusd_admin_port",rules.is_number, description=u"Radius服务管理端口",**input_style),
    pyforms.Textbox("acct_interim_intelval",rules.is_number, description=u"Radius记账间隔(秒)",**input_style),
    pyforms.Textbox("max_session_timeout",rules.is_number, description=u"Radius最大会话时长(秒)", **input_style),
    pyforms.Textbox("reject_delay",rules.is_number, description=u"拒绝延迟时间(秒)(0-9)",**input_style),
    pyforms.Textbox("portal_secret", description=u"portal登陆密钥", **input_style),
    pyforms.Button("submit", type="submit", html=u"<b>更新</b>", **button_style),
    title=u"参数配置管理",
    action="/param/update"
)


passwd_update_form = pyforms.Form(
    pyforms.Textbox("operator_name", description=u"管理员名", size=32, readonly="readonly", **input_style),
    pyforms.Password("operator_pass", rules.len_of(6, 32), description=u"管理员新密码", size=32,value="", required="required", **input_style),
    pyforms.Password("operator_pass_chk", rules.len_of(6, 32), description=u"确认管理员新密码", size=32,value="", required="required", **input_style),
    pyforms.Button("submit", type="submit", html=u"<b>更新</b>", **button_style),
    title=u"管理密码更新",
    action="/passwd"
)

node_add_form = pyforms.Form(
    pyforms.Textbox("node_name", rules.len_of(2,32), description=u"区域名称",required="required",**input_style),
    pyforms.Textarea("node_desc", rules.len_of(0, 128), description=u"区域描述",**input_style),
    pyforms.Button("submit", type="submit", html=u"<b>提交</b>",**button_style),
    title=u"增加区域",
    action="/node/add"
)

node_update_form = pyforms.Form(
    pyforms.Hidden("id",  description=u"区域编号"),
    pyforms.Textbox("node_name", rules.len_of(2, 32), description=u"区域名称", required="required",**input_style),
    pyforms.Textarea("node_desc", rules.len_of(0, 128), description=u"区域描述", **input_style),
    pyforms.Button("submit", type="submit", html=u"<b>更新</b>", **button_style),
    title=u"修改区域",
    action="/node/update"
)

timetype = {0:u"标准时区,北京时间",1:u"时区和时间同区"}
bastype = {
            '0' : u'标准',
            '9' : u'思科',
            '3041' : u'阿尔卡特',
            '2352' : u'爱立信',
            '2011' : u'华为',
            '25506' : u'H3C',
            '3902' : u'中兴',
            '10055' : u'爱快',
            '14988' : u'RouterOS'
        }

bas_add_form = pyforms.Form(
        pyforms.Textbox("ip_addr", rules.is_ip, description=u"BAS地址",required="required",**input_style),
        pyforms.Textbox("bas_name", rules.len_of(2,64), description=u"BAS名称",required="required",**input_style),
        pyforms.Textbox("bas_secret", rules.is_alphanum2(4, 32), description=u"共享秘钥", required="required",**input_style),
        pyforms.Dropdown("vendor_id", description=u"BAS类型", args=bastype.items(), required="required",**input_style),
        pyforms.Textbox("coa_port", rules.is_number,description=u"CoA端口", default=3799,required="required",**input_style),
        pyforms.Dropdown("time_type", description=u"时间类型", args=timetype.items(), required="required",**input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"增加BAS",
        action="/bas/add"
    )

bas_update_form = pyforms.Form(
        pyforms.Hidden("id",  description=u"编号"),
        pyforms.Textbox("ip_addr", rules.is_ip, description=u"BAS地址",  readonly="readonly",**input_style),
        pyforms.Textbox("bas_name", rules.len_of(2,64), description=u"BAS名称", required="required",**input_style),
        pyforms.Textbox("bas_secret", rules.is_alphanum2(4, 32), description=u"共享秘钥", required="required",**input_style),
        pyforms.Dropdown("vendor_id", description=u"BAS类型", args=bastype.items(), required="required",**input_style),
        pyforms.Textbox("coa_port", rules.is_number,description=u"CoA端口", default=3799,required="required",**input_style),
        pyforms.Dropdown("time_type", description=u"时间类型", args=timetype.items(), required="required",**input_style),
        pyforms.Button("submit", type="submit", html=u"<b>更新</b>", **button_style),
        title=u"修改BAS",
        action="/bas/update"
    )
    
opr_type = {0:u'系统管理员',1:u"普通操作员"}    
opr_status_dict = {0:u'正常',1:u"停用"}
    
def opr_add_form(nodes=[]):
    return pyforms.Form(
        pyforms.Textbox("operator_name", rules.len_of(2,32), description=u"操作员名称",required="required",**input_style),
        pyforms.Textbox("operator_desc", rules.len_of(0,255),description=u"操作员姓名",**input_style),
        pyforms.Password("operator_pass", rules.len_of(6, 128), description=u"操作员密码", required="required",**input_style),
        pyforms.Dropdown("operator_status", description=u"操作员状态", args=opr_status_dict.items(), required="required",**input_style),
        pyforms.Dropdown("operator_nodes", description=u"关联区域(多选)",args=nodes, required="required",multiple="multiple",size=4,**input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"增加操作员",
        action="/opr/add"
    )  
    
def opr_update_form(nodes=[]):
    return pyforms.Form(
        pyforms.Hidden("id",  description=u"编号"),
        pyforms.Textbox("operator_name", description=u"操作员名称",readonly="readonly",**input_style),
        pyforms.Textbox("operator_desc", rules.len_of(0,255),description=u"操作员姓名",**input_style),
        pyforms.Password("operator_pass", rules.len_of(0, 128), description=u"操作员密码(留空不修改)",**input_style),
        pyforms.Dropdown("operator_status", description=u"操作员状态", args=opr_status_dict.items(), required="required",**input_style),
        pyforms.Dropdown("operator_nodes", description=u"关联区域(多选)",args=nodes, required="required",multiple="multiple",size=4,**input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"修改操作员",
        action="/opr/update"
    )        

product_policy = {0:u'预付费包月',1:u"预付费时长",2:u"买断包月",3:u"买断时长",4:u"预付费流量",5:u"买断流量"}
product_status_dict = {0:u'正常',1:u"停用"}

def product_add_form():
    return pyforms.Form(
        pyforms.Textbox("product_name", rules.len_of(4, 64), description=u"资费名称",  required="required",**input_style),
        pyforms.Dropdown("product_policy", args=product_policy.items(), description=u"计费策略", required="required",**input_style),
        pyforms.Textbox("fee_months", rules.is_number, description=u"买断授权月数",value=0, **input_style),
        pyforms.Textbox("fee_times", rules.is_number, description=u"买断时长(小时)",value=0, **input_style),
        pyforms.Textbox("fee_flows", rules.is_number, description=u"买断流量(MB)",value=0, **input_style),
        pyforms.Textbox("fee_price", rules.is_rmb, description=u"资费价格(元)", required="required", **input_style),
        pyforms.Hidden("fee_period",rules.is_period,description=u"开放认证时段",**input_style),
        pyforms.Textbox("concur_number", rules.is_numberOboveZore,description=u"并发数控制(0表示不限制)",value="0", **input_style),
        pyforms.Dropdown("bind_mac",  args=boolean.items(), description=u"是否绑定MAC ",**input_style),
        pyforms.Dropdown("bind_vlan",  args=boolean.items(),description=u"是否绑定VLAN ",**input_style),
        pyforms.Textbox("input_max_limit",  rules.is_number3,description=u"最大上行速率(Mbps)",required="required", **input_style),
        pyforms.Textbox("output_max_limit",  rules.is_number3,description=u"最大下行速率(Mbps)",required="required",**input_style),
        pyforms.Dropdown("product_status", args=product_status_dict.items(),description=u"资费状态", required="required", **input_style),
        pyforms.Button("submit", type="submit",id="submit",html=u"<b>提交</b>", **button_style),
        title=u"增加资费",
        action="/product/add"
    )

def product_update_form():
    return pyforms.Form(
        pyforms.Hidden("id",  description=u"编号"),
        pyforms.Hidden("product_policy",  description=u""),
        pyforms.Textbox("product_name", rules.len_of(4, 32), description=u"资费名称", required="required",**input_style),
        pyforms.Textbox("product_policy_name", description=u"资费策略",readonly="readonly", required="required",**input_style),
        pyforms.Dropdown("product_status",args=product_status_dict.items(), description=u"资费状态", required="required", **input_style),
        pyforms.Textbox("fee_months", rules.is_number, description=u"买断授权月数",value=0, **input_style),
        pyforms.Textbox("fee_times", rules.is_number, description=u"买断时长(小时)",value=0, **input_style),
        pyforms.Textbox("fee_flows", rules.is_number, description=u"买断流量(MB)",value=0, **input_style),
        pyforms.Textbox("fee_price", rules.is_rmb,description=u"资费价格(元)", required="required", **input_style),
        pyforms.Hidden("fee_period", rules.is_period,description=u"开放认证时段",**input_style),
        pyforms.Textbox("concur_number", rules.is_number,description=u"并发数控制(0表示不限制)",required="required", **input_style),
        pyforms.Dropdown("bind_mac",  args=boolean.items(), description=u"是否绑定MAC",required="required",**input_style),
        pyforms.Dropdown("bind_vlan",  args=boolean.items(),description=u"是否绑定VLAN",required="required",**input_style),
        pyforms.Textbox("input_max_limit",  rules.is_number3,description=u"最大上行速率(Mbps)",required="required", **input_style),
        pyforms.Textbox("output_max_limit",  rules.is_number3,description=u"最大下行速率(Mbps)",required="required",**input_style),
        pyforms.Button("submit", type="submit", html=u"<b>更新</b>", **button_style),
        title=u"修改资费",
        action="/product/update"
    )

product_attr_add_form = pyforms.Form(
    pyforms.Hidden("product_id",  description=u"资费编号"),
    pyforms.Textbox("attr_name", rules.len_of(1, 255), description=u"策略名称",  required="required",help=u"策略参考",**input_style),
    pyforms.Textbox("attr_value", rules.len_of(1, 255),description=u"策略值", required="required",**input_style),
    pyforms.Textbox("attr_desc", rules.len_of(1, 255),description=u"策略描述", required="required",**input_style),
    pyforms.Button("submit", type="submit", html=u"<b>提交</b>", **button_style),
    title=u"增加策略属性",
    action="/product/attr/add"
)

product_attr_update_form = pyforms.Form(
    pyforms.Hidden("id",  description=u"编号"),
    pyforms.Hidden("product_id",  description=u"资费编号"),
    pyforms.Textbox("attr_name", rules.len_of(1, 255), description=u"策略名称",  readonly="required",**input_style),
    pyforms.Textbox("attr_value", rules.len_of(1, 255),description=u"策略值", required="required",**input_style),
    pyforms.Textbox("attr_desc", rules.len_of(1, 255),description=u"策略描述", required="required",**input_style),
    pyforms.Button("submit", type="submit", html=u"<b>更新</b>", **button_style),
    title=u"修改策略属性",
    action="/product/attr/update"
)


roster_type = {0:u"白名单", 1:u"黑名单"}

roster_add_form = pyforms.Form(
        pyforms.Textbox("mac_addr", description=u"MAC地址",required="required",**input_style),
        pyforms.Textbox("begin_time",description=u"开始时间", required="required",**input_style),
        pyforms.Textbox("end_time", description=u"结束时间", required="required",**input_style),
        pyforms.Dropdown("roster_type", args=roster_type.items(),description=u"类型",value=0, required="required",**input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"增加黑白名单",
        action="/roster/add"
    )

roster_update_form = pyforms.Form(
    pyforms.Hidden("id",  description=u"编号"),
    pyforms.Textbox("mac_addr", description=u"MAC地址",readonly="readonly",**input_style),
    pyforms.Textbox("begin_time",description=u"开始时间", required="required",**input_style),
    pyforms.Textbox("end_time", description=u"结束时间", required="required",**input_style),
    pyforms.Dropdown("roster_type", args=roster_type.items(),description=u"类型",value=0, required="required",**input_style),
    pyforms.Button("submit",  type="submit", html=u"<b>更新</b>", **button_style),
    title=u"修改黑白名单",
    action="/roster/update"
)


user_state = {1:u"正常", 2:u"停机" , 3:u"销户", 4:u"到期"}
bind_state = {0: u"不绑定", 1: u"绑定"}

def user_open_form(nodes=[],products=[]):
    return pyforms.Form(
        pyforms.Dropdown("node_id", description=u"区域", args=nodes,required="required", **input_style),
        pyforms.Textbox("realname", rules.len_of(2,32), description=u"用户姓名", required="required",**input_style),
        pyforms.Checkbox("is_samename",description=u"启用独立的自助服务用户名", checked=""),
        pyforms.Textbox("member_name", rules.len_of(0,64), description=u"自助服务用户名", **input_style),
        pyforms.Textbox("member_password", rules.len_of(0,128), description=u"自助服务用户密码", **input_style),
        pyforms.Textbox("idcard", rules.len_of(0,32), description=u"证件号码", **input_style),
        pyforms.Textbox("mobile", rules.len_of(0,32),description=u"用户手机号码", **input_style),
        pyforms.Textbox("address", description=u"用户地址",hr=True, **input_style),
        pyforms.Textbox("account_number", description=u"用户账号",  required="required", **input_style),
        pyforms.Textbox("password", description=u"认证密码", required="required", **input_style),
        pyforms.Textbox("ip_address", description=u"用户IP地址",**input_style),
        pyforms.Dropdown("product_id",args=products, description=u"资费",  required="required", **input_style),
        pyforms.Textbox("months",rules.is_number, description=u"月数(包月有效)", required="required", **input_style),
        pyforms.Textbox("fee_value",rules.is_rmb, description=u"缴费金额",  required="required", **input_style),
        pyforms.Textbox("expire_date", rules.is_date,description=u"过期日期",  required="required", **input_style),
        pyforms.Hidden("status",value=1, description=u"用户状态",  **input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"用户开户",
        action="/bus/member/open"
    )

def account_open_form(products=[]):
    return pyforms.Form(
        pyforms.Hidden("node_id", description=u"区域", **input_style),
        pyforms.Hidden("member_id",  description=u"编号"),
        pyforms.Textbox("realname", description=u"用户姓名", readonly="readonly",**input_style),
        pyforms.Textbox("account_number", description=u"用户账号",  required="required", **input_style),
        pyforms.Textbox("password", description=u"认证密码", required="required", **input_style),
        pyforms.Textbox("ip_address", description=u"用户IP地址",**input_style),
        pyforms.Textbox("address", description=u"用户装机地址",**input_style),
        pyforms.Dropdown("product_id",args=products, description=u"资费",  required="required", **input_style),
        pyforms.Textbox("months",rules.is_number, description=u"月数(包月有效)", required="required", **input_style),
        pyforms.Textbox("fee_value",rules.is_rmb, description=u"缴费金额",  required="required", **input_style),
        pyforms.Textbox("expire_date", rules.is_date,description=u"过期日期",  required="required", **input_style),
        pyforms.Hidden("status",value=1, description=u"用户状态",  **input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"用户新开账号",
        action="/bus/account/open"
    )

def user_import_form(nodes=[],products=[]):
    return pyforms.Form(
        pyforms.Dropdown("node_id", description=u"用户区域", args=nodes, **input_style),
        pyforms.Dropdown("product_id",args=products, description=u"用户资费",  required="required", **input_style),
        pyforms.File("import_file", description=u"用户数据文件",  required="required", **input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>立即导入</b>", **button_style),
        title=u"用户数据导入",
        action="/bus/member/import"
)

user_import_vform = dataform.Form(
        dataform.Item("realname", rules.not_null,description=u"用户姓名" ),
        dataform.Item("idcard", rules.len_of(0,32), description=u"证件号码"),
        dataform.Item("mobile", rules.len_of(0,32),description=u"用户手机号码"),
        dataform.Item("address", description=u"用户地址"),
        dataform.Item("account_number",rules.not_null, description=u"用户账号"),
        dataform.Item("password",rules.not_null,description=u"用户密码"),
        dataform.Item("begin_date", rules.is_date,description=u"开通日期"),
        dataform.Item("expire_date", rules.is_date,description=u"过期日期"),
        dataform.Item("balance",rules.is_rmb,description=u"用户余额"),
        dataform.Item("time_length",description=u"用户时长"),
        dataform.Item("flow_length",description=u"用户流量"),
        title="import"
)

account_next_form = pyforms.Form(
        pyforms.Hidden("product_id", description=u"资费"),
        pyforms.Hidden("old_expire", description=u""),
        pyforms.Hidden("account_number", description=u"用户账号"),
        pyforms.Textbox("months",rules.is_number, description=u"月数(包月有效)",value=0, required="required", **input_style),
        pyforms.Textbox("fee_value",rules.is_rmb, description=u"缴费金额",value=0,  required="required", **input_style),
        pyforms.Textbox("expire_date", rules.is_date,description=u"过期日期",  required="required", **input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"用户续费",
        action="/bus/account/next"
)

account_charge_form = pyforms.Form(
        pyforms.Hidden("account_number", description=u"用户账号",  required="required", **input_style),
        pyforms.Textbox("fee_value",rules.is_rmb, description=u"缴费金额", value=0, required="required", **input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"用户充值",
        action="/bus/account/charge"
)

account_cancel_form = pyforms.Form(
        pyforms.Hidden("account_number", description=u"用户账号",  required="required", **input_style),
        pyforms.Textbox("fee_value",rules.is_rmb, description=u"退费金额",  required="required", **input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"用户销户",
        action="/bus/account/cancel"
)


def account_change_form(products=[]):
    return pyforms.Form(
        pyforms.Hidden("account_number", description=u"用户账号",  required="required", **input_style),
        pyforms.Dropdown("product_id",args=products, description=u"资费",  required="required", **input_style),
        pyforms.Textbox("add_value",rules.is_rmb, description=u"缴费金额",required="required",value="0", **input_style),
        pyforms.Textbox("back_value",rules.is_rmb, description=u"退费金额",required="required",value="0", **input_style),
        pyforms.Textbox("expire_date", rules.is_date,description=u"过期日期",value="0000-00-00",**input_style),
        pyforms.Textbox("balance",rules.is_rmb,description=u"用户变更后余额",value="0.00",**input_style),
        pyforms.Textbox("time_length",description=u"用户时长(小时)",value="0.00",**input_style),
        pyforms.Textbox("flow_length",description=u"用户流量(MB)",value="0",**input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"用户资费变更",
        action="/bus/account/change"
)

def member_update_form(nodes=[]):
    return pyforms.Form(
        pyforms.Hidden("account_number", description=u"用户账号"),
        pyforms.Hidden("member_id", description=u"mid",  required="required", **input_style),
        pyforms.Textbox("realname", rules.len_of(2,32), description=u"用户姓名", required="required",**input_style),
        pyforms.Textbox("member_name", description=u"自助服务用户名", readonly="readonly",**input_style),
        pyforms.Password("new_password", rules.len_of(0,128),value="", description=u"自助服务密码(留空不修改)", **input_style),
        pyforms.Textbox("email", rules.len_of(0,128), description=u"电子邮箱", **input_style),
        pyforms.Textbox("idcard", rules.len_of(0,32), description=u"证件号码", **input_style),
        pyforms.Textbox("mobile", rules.len_of(0,32),description=u"用户手机号码", **input_style),
        pyforms.Textbox("address", description=u"用户地址",hr=True, **input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"用户基本信息修改",
        action="/bus/member/update"
    )


def account_update_form():
    return pyforms.Form(
        pyforms.Textbox("account_number", description=u"用户账号",  readonly="readonly", **input_style),
        pyforms.Textbox("ip_address", description=u"用户IP地址",**input_style),
        pyforms.Hidden("install_address", description=u"用户安装地址",**input_style),
        pyforms.Textbox("new_password", description=u"认证密码(留空不修改)", **input_style),
        pyforms.Textbox("user_concur_number",rules.is_number, description=u"用户并发数",  required="required", **input_style),
        pyforms.Dropdown("bind_mac",  args=boolean.items(), description=u"是否绑定MAC",**input_style),
        pyforms.Dropdown("bind_vlan",  args=boolean.items(),description=u"是否绑定VLAN",**input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"用户变更资料",
        action="/bus/account/update"
    )

card_types = {0:u'资费卡',1:u'余额卡'}
card_states = {0:u'未激活',1:u'已激活',2:u"已使用",3:u"已回收"}

def recharge_card_form(products=[]):
    return pyforms.Form(
        pyforms.Dropdown("card_type",  args=card_types.items(), description=u"充值卡类型",**input_style),
        pyforms.Textbox("batch_no", rules.is_number,maxlength=8,description=u"批次号(年+月+2位序号，如：20150201)",required="required", **input_style),
        pyforms.Dropdown("product_id",args=products, description=u"资费",**input_style),
        pyforms.Textbox("start_no",rules.is_number,maxlength=5, description=u"开始卡号(最大5位)",required="required",**input_style),
        pyforms.Textbox("stop_no", rules.is_number,maxlength=5,description=u"结束卡号(最大5位)",required="required",**input_style),
        pyforms.Textbox("pwd_len", rules.is_number,description=u"密码长度(最大为16)",value=8,**input_style),
        pyforms.Textbox("fee_value",rules.is_rmb, description=u"面值/销售价(元)",value=0,**input_style),
        pyforms.Textbox("months", rules.is_number,description=u"授权时间(月)",readonly="readonly",value=0,**input_style),
        pyforms.Textbox("times",description=u"总时长(小时)",readonly="readonly",value=0,**input_style),
        pyforms.Textbox("flows",description=u"总流量(MB)",readonly="readonly",value=0,**input_style),
        pyforms.Textbox("expire_date",rules.is_date, description=u"过期时间",required="required",**input_style),
        pyforms.Button("submit",  type="submit", html=u"<b>提交</b>", **button_style),
        title=u"充值卡生成",
        action="/card/create"
    )
    










#!/usr/bin/env python
#coding:utf-8
import sys,os
from twisted.python import log
from twisted.internet import reactor
from twisted.web import server, wsgi
from twisted.python.logfile import DailyLogFile
from bottle import request
from bottle import response
from toughradius.console.customer.customer import app as mainapp
from toughradius.console.libs import sqla_plugin,utils
from toughradius.console.libs.smail import mail
from toughradius.console.websock import websock
from toughradius.console import base
from toughradius.console import models
from toughradius.tools.dbengine import get_engine
from toughradius.console.base import (
    get_cookie,
    set_cookie,
    get_param_value,
    get_member_by_name,
    get_account_by_number,
    get_online_status,
    Connect,
    Render
)
import functools
import time
import bottle

reactor.suggestThreadPoolSize(30)


###############################################################################
# run server                                                                 
###############################################################################

class CustomerServer(object):
    
    def __init__(self,config,db_engine=None,daemon=False,app=None,subapps=[]):
        self.config = config
        self.app = app
        self.subapps = subapps
        self.db_engine = db_engine
        self.daemon = daemon
        self.viewpath = os.path.join(os.path.split(__file__)[0],"customer/views/")
        self.init_config()
        self.init_timezone()
        self.init_db_engine()
        self.init_application()
        self.init_websock()
        self.init_mail()
        self.init_protocol()
    
    def init_config(self):
        self.logfile = self.config.get('customer','logfile')
        self.standalone = self.config.has_option('DEFAULT','standalone') and \
            self.config.getboolean('DEFAULT','standalone') or False
        self.secret = self.config.get('DEFAULT','secret')
        self.timezone = self.config.has_option('DEFAULT','tz') and self.config.get('DEFAULT','tz') or "CST-8"
        self.debug = self.config.getboolean('DEFAULT','debug')
        self.port = self.config.getint('customer','port')
        self.host = self.config.has_option('customer','host') \
            and self.config.get('customer','host') or  '0.0.0.0'
        bottle.debug(self.debug)
        base.scookie.setup(self.secret)
        # update aescipher
        utils.aescipher.setup(self.secret)
        self.encrypt = utils.aescipher.encrypt
        self.decrypt = utils.aescipher.decrypt
        #parse ssl
        self._check_ssl_config()
        
    def _check_ssl_config(self):
        self.use_ssl = False
        self.privatekey = None
        self.certificate = None
        if self.config.has_option('DEFAULT','ssl') and self.config.getboolean('DEFAULT','ssl'):
            self.privatekey = self.config.get('DEFAULT','privatekey')
            self.certificate = self.config.get('DEFAULT','certificate')
            if os.path.exists(self.privatekey) and os.path.exists(self.certificate):
                self.use_ssl = True
                
    def init_timezone(self):
        try:
            os.environ["TZ"] = self.timezone
            time.tzset()
        except:pass
    
    def init_db_engine(self):
        if not self.db_engine:
            self.db_engine = get_engine(self.config)
        metadata = models.get_metadata(self.db_engine)
        self.sqla_pg = sqla_plugin.Plugin(
            self.db_engine,
            metadata,
            keyword='db',
            create=False,
            commit=False,
            use_kwargs=False
        )
        
    def init_protocol(self):
        self.web_factory = server.Site(wsgi.WSGIResource(reactor, reactor.getThreadPool(), self.app))
        
    def _sys_param_value(self,pname):
        with Connect(self.sqla_pg.new_session) as db:
            return get_param_value(db,pname)
     
    def _get_product_name(self,pid):
        with Connect(self.sqla_pg.new_session) as db:
            return get_product_name(db,pid)
            
    def _get_member_by_name(self,mname):
        with Connect(self.sqla_pg.new_session) as db:
            return get_member_by_name(db,mname)
    
    def _get_account_by_number(self,account):
        with Connect(self.sqla_pg.new_session) as db:
            return get_account_by_number(db,account)
            
    def _get_online_status(self,account):
        with Connect(self.sqla_pg.new_session) as db:
            return get_online_status(db,account)
            
    def error403(self,error):
        return self.render.render("error",msg=u"Unauthorized access %s"%error.exception)
    
    def error404(self,error):
        return self.render.render("error",msg=u"Not found %s"%error.exception)

    def error500(self,error):
        return self.render.render("error",msg=u"Server Internal error %s"%error.exception)
        
    def init_application(self):
        log.msg("start init application...")
        _lookup = [self.viewpath]
        _context = dict(
            use_ssl = self.use_ssl,
            get_cookie = get_cookie,
            fen2yuan = utils.fen2yuan,
            fmt_second = utils.fmt_second,
            bb2mb = utils.bb2mb,
            bbgb2mb = utils.bbgb2mb,
            kb2mb = utils.kb2mb,
            mb2kb = utils.mb2kb,
            sec2hour = utils.sec2hour,
            request = request,
            sys_param_value = self._sys_param_value,
            get_member = self._get_member_by_name,
            get_account = self._get_account_by_number,
            is_online = self._get_online_status
        )
        self.render = Render(_context,_lookup)
        log.msg("mount app and install plugins...")
        for _app in [self.app]+self.subapps:
            _app.config['render_key'] = 'customer'
            Render.RENDERS['customer'] = self.render
            _app.error_handler[403] = self.error403
            _app.error_handler[404] = self.error404
            _app.error_handler[500] = self.error500
            _app.install(self.sqla_pg)
            if _app is not self.app:
                self.app.mount(_app.config['__prefix__'],_app)
        
    def init_mail(self):
        log.msg("init sendmail..")
        mail.setup(
            server=self._sys_param_value('smtp_server'),
            user=self._sys_param_value('smtp_user'),
            pwd=self._sys_param_value('smtp_pwd'),
            fromaddr=self._sys_param_value('smtp_user'),
            sender=self._sys_param_value('smtp_sender')
        )
        
    def init_websock(self):
        # connect radiusd websocket admin port 
        log.msg("init websocket client...")
        websock.use_ssl = self.use_ssl
        wsparam = (
                self._sys_param_value('radiusd_address'),
                self._sys_param_value('radiusd_admin_port')
        )
        reactor.callLater(1, websock.connect,*wsparam)
    
    def run_normal(self):
        if self.debug:
            log.startLogging(sys.stdout)
        else:
            log.startLogging(DailyLogFile.fromFullPath(self.logfile))
        log.msg('server listen %s'%self.host)
        if self.use_ssl:
            log.msg('Customer SSL Enable!')
            from twisted.internet import ssl
            sslContext = ssl.DefaultOpenSSLContextFactory(self.privatekey, self.certificate)
            reactor.listenSSL(
                self.port,
                self.web_factory,
                contextFactory = sslContext,
                interface=self.host
            )
        else:
            reactor.listenTCP(self.port, self.web_factory,interface=self.host)
        if not self.standalone:
            reactor.run()
        
    def get_service(self):
        from twisted.application import service, internet
        if self.use_ssl:
            log.msg('Admin SSL Enable!')
            from twisted.internet import ssl
            sslContext = ssl.DefaultOpenSSLContextFactory(self.privatekey, self.certificate)
            return internet.SSLServer(
                self.port,
                self.web_factory,
                contextFactory = sslContext,
                interface = self.host
            )
        else: 
            log.msg('Admin SSL Disable!')       
            return internet.TCPServer(self.port,self.web_factory,interface = self.host)    
 
def run(config,db_engine=None,is_service=False):
    print 'running customer server...'
    admin = CustomerServer(config,db_engine,daemon=is_service,app=mainapp)
    if is_service:
        return admin.get_service()
    else:
        admin.run_normal()
            



#!/usr/bin/env python
#coding=utf-8
from toughradius.radiusd.plugins import error_auth

def process(req=None,resp=None,user=None,radiusd=None,**kwargs):
    """check mac bind & vlan bind"""
    store = radiusd.store
    macaddr = req.get_mac_addr()
    if store.is_white_roster(macaddr):
        return resp
        
    if macaddr and  user['mac_addr']:
        if user['bind_mac'] == 1 and macaddr not in user['mac_addr']:
            return error_auth(resp,"macaddr bind not match")
    elif macaddr and not user['mac_addr'] :
        store.update_user_mac(user['account_number'], macaddr)

    vlan_id,vlan_id2 = req.get_vlanids()
    print vlan_id,vlan_id2
    if vlan_id and user['vlan_id']:
        if user['bind_vlan'] == 1 and vlan_id != user['vlan_id']:
            return error_auth(resp,"vlan_id bind not match")
    elif vlan_id and not user['vlan_id']:
        user['vlan_id'] = vlan_id
        store.update_user_vlan_id(user['account_number'],vlan_id)

    if vlan_id2 and user['vlan_id2']:
        if user['bind_vlan'] == 1 and vlan_id2 != user['vlan_id2']:
            return error_auth(resp,"vlan_id2 bind not match")
    elif vlan_id2 and not user['vlan_id2']:
        user['vlan_id2'] = vlan_id2
        store.update_user_vlan_id2(user['account_number'],vlan_id2)

    return resp


#!/usr/bin/env python
#coding=utf-8
import copy
from toughradius.console.libs.pyforms import  attrget
from toughradius.console.libs.pyforms import storage
from toughradius.console.libs.pyforms import AttributeList

class Form(object):

    def __init__(self, *inputs, **kw):
        self.inputs = inputs
        self.valid = True
        self.note = None
        self.validators = kw.pop('validators', [])

    def __call__(self, x=None):
        o = copy.deepcopy(self)
        if x: o.validates(x)
        return o

    @property
    def errors(self):
        return ",".join([u"%s error,%s" % (i.description, i.note) for i in self.inputs if i.note])

    def validates(self, source=None, _validate=True, **kw):
        source = source or kw
        out = True
        for i in self.inputs:
            v = attrget(source, i.name)
            if _validate:
                out = i.validate(v) and out
            else:
                i.set_value(v)
        if _validate:
            out = out and self._validate(source)
            self.valid = out
        return out

    def _validate(self, value):
        self.value = value
        for v in self.validators:
            if not v.valid(value):
                self.note = v.msg
                return False
        return True

    def fill(self, source=None, **kw):
        return self.validates(source, _validate=False, **kw)

    def __getitem__(self, i):
        for x in self.inputs:
            if x.name == i: return x
        raise KeyError, i

    def __getattr__(self, name):
        # don't interfere with deepcopy
        inputs = self.__dict__.get('inputs') or []
        for x in inputs:
            if x.name == name: return x
        raise AttributeError, name

    def get(self, i, default=None):
        try:
            return self[i]
        except KeyError:
            return default

    def _get_d(self):  #@@ should really be form.attr, no?
        return storage([(i.name, i.get_value()) for i in self.inputs])

    d = property(_get_d)


class Item(object):
    def __init__(self, name, *validators, **attrs):
        self.name = name
        self.validators = validators
        self.attrs = attrs = AttributeList(attrs)

        self.description = attrs.pop('description', name)
        self.value = attrs.pop('value', None)
        self.note = None

        self.id = attrs.setdefault('id', self.get_default_id())


    def get_default_id(self):
        return self.name

    def validate(self, value):
        self.set_value(value)

        for v in self.validators:
            if not v.valid(value):
                self.note = v.msg
                return False
        return True

    def set_value(self, value):
        self.value = value

    def get_value(self):
        return self.value

    def addatts(self):
        # add leading space for backward-compatibility
        return " " + str(self.attrs)


#!/usr/bin/env python
#coding=utf-8
from twisted.python import log
from toughradius.radiusd.settings import *
import logging


def process(req=None,user=None,radiusd=None,**kwargs):
    if  req.get_acct_status_type() not in (STATUS_TYPE_ACCT_ON,STATUS_TYPE_ACCT_OFF):
        return
    
    runstat=radiusd.runstat
    store = radiusd.store

    if req.get_acct_status_type() == STATUS_TYPE_ACCT_ON:
        store.unlock_online(req.get_nas_addr(),None,STATUS_TYPE_ACCT_ON)
        runstat.acct_on += 1  
        log.msg('bas accounting on success',level=logging.INFO)
    else:
        store.unlock_online(req.get_nas_addr(),None,STATUS_TYPE_ACCT_OFF)
        runstat.acct_off += 1  
        log.msg('bas accounting off success',level=logging.INFO)


#!/usr/bin/env python
#coding:utf-8
from twisted.python import log
from bottle import request
from bottle import response
from bottle import redirect
from bottle import HTTPError
from bottle import static_file
from bottle import mako_template
from toughradius.console.libs.paginator import Paginator
from toughradius.console.libs import utils
from toughradius.console import models
from beaker.cache import CacheManager
from hashlib import md5
import logging
import functools
import urllib
import json
import time
import tempfile

########################################################################
# const define
########################################################################

FEES = (PPMonth,PPTimes,BOMonth,BOTimes,PPFlow,BOFlows) = (0,1,2,3,4,5)

ACCOUNT_STATUS = (UsrPreAuth,UsrNormal,UsrPause,UsrCancel,UsrExpire) = (0,1,2,3,4)

CARD_STATUS = (CardInActive,CardActive,CardUsed,CardRecover) = (0,1,2,3)

CARD_TYPE = (ProductCard,BalanceCard) = (0,1)

ACCEPT_TYPES = {
    'open':u'开户',
    'pause':u'停机',
    'resume':u'复机',
    'cancel':u'销户',
    'next':u'续费',
    'charge':u'充值',
    'change':u'变更'
}

MAX_EXPIRE_DATE = '3000-12-30'

TMPDIR = tempfile.gettempdir()

page_size = 20

__cache_timeout__ = 600

cache = CacheManager(cache_regions={'short_term':{ 'type': 'memory', 'expire': __cache_timeout__ }}) 
   
class Connect:
    def __init__(self, mkdb):
        self.conn = mkdb()

    def __enter__(self):
        return self.conn   

    def __exit__(self, exc_type, exc_value, exc_tb):
        self.conn.close()

class SecureCookie(object):
    
    def setup(self,secret):
        self.secret = secret
    
    def get_cookie(self,name):
        return request.get_cookie(md5(name).hexdigest(),secret=self.secret)
        
    def set_cookie(self,name,value,**options):
        response.set_cookie(md5(name).hexdigest(),value,secret=self.secret,**options)
        
scookie = SecureCookie()
get_cookie = scookie.get_cookie
set_cookie = scookie.set_cookie

class Render(object):
    
    RENDERS = {}
    
    def __init__(self,context={},lookup=None):
        self.context = context
        self.lookup = lookup

    def render(self,*args,**kwargs):
        kwargs['template_lookup'] = self.lookup
        kwargs.update(**self.context)
        return mako_template(*args,**kwargs)
    
    @staticmethod 
    def render_app(app,*args,**kwargs):
        render_obj = Render.RENDERS[app.config['render_key']]
        return render_obj.render(*args,**kwargs)
        
########################################################################
# permission manage
########################################################################

class Permit():
    routes = {}
    def add_route(self,path,name,category,is_menu=False,order=time.time(),is_open=True):
        if not path:return
        self.routes[path] = dict(
            path=path,
            name=name,
            category=category,
            is_menu=is_menu,
            oprs=[],
            order=order,
            is_open=is_open
        )
    
    def get_route(self,path):
        return self.routes.get(path)    
        
    def bind_super(self,opr):
        for path in self.routes:
            route = self.routes.get(path)    
            route['oprs'].append(opr)         
    
    def bind_opr(self,opr,path):
        if not path or path not in self.routes:
            return
        oprs = self.routes[path]['oprs'] 
        if opr not in oprs:
            oprs.append(opr) 
            
    def unbind_opr(self,opr,path=None):
        if path:
            self.routes[path]['oprs'].remove(opr)
        else:
            for path in self.routes:
                route = self.routes.get(path)    
                if route and opr in route['oprs']:
                    route['oprs'].remove(opr)
                    
    def check_open(self,path):
        route = self.routes[path]
        return 'is_open' in route and route['is_open']
                
    def check_opr_category(self,opr,category):
        for path in self.routes:
            route = self.routes[path]
            if opr in route['oprs'] and route['category'] == category:
                return True
        return False 
        
    def build_menus(self,order_cats=[]):
        menus = [ {'category':_cat,'items':[]} for _cat in order_cats]
        for path in self.routes:
            route = self.routes[path]
            for menu in menus:
                if route['category'] == menu['category']:
                    menu['items'].append(route)
        return menus
        
    def match(self,path):
        if not path:
            return False
        return get_cookie("username") in self.routes[path]['oprs']
     
permit = Permit()       

########################################################################
# web plugin funtion
########################################################################

def export_file(name,data):
    with open(u'%s/%s' % (TMPDIR,name), 'wb') as f:
        f.write(data.xls)
    return static_file(name, root=TMPDIR,download=True)

def auth_opr(func):
    @functools.wraps(func)
    def warp(*args,**kargs):
        if not get_cookie("username"):
            log.msg("admin login timeout")
            return redirect('/login')
        else:
            opr = get_cookie("username")
            rule = permit.get_route(request.fullpath)
            if rule and opr not in rule['oprs']:
                raise HTTPError(403, u'访问被拒绝')
            return func(*args,**kargs)
    return warp
    
def auth_cus(func):
    @functools.wraps(func)
    def warp(*args,**kargs):
        if not get_cookie("customer"):
            log.msg("user login timeout")
            return redirect('/login')
        else:
            return func(*args,**kargs)
    return warp    

########################################################################
# cache function
########################################################################
def get_opr_nodes(db):
    opr_type = get_cookie('opr_type')
    if opr_type == 0:
        return db.query(models.SlcNode)
    opr_name = get_cookie('username')
    return db.query(models.SlcNode).filter(
        models.SlcNode.node_name == models.SlcOperatorNodes.node_name,
        models.SlcOperatorNodes.operator_name == opr_name
    )

@cache.cache('get_node_name',expire=3600)   
def get_node_name(db,node_id):
    return  db.query(models.SlcNode.node_name).filter_by(id=node_id).scalar()
    
@cache.cache('get_product_name',expire=3600)   
def get_product_name(db,product_id):
    if not product_id:return
    return  db.query(models.SlcRadProduct.product_name).filter_by(id=product_id).scalar()

@cache.cache('get_account_node_id',expire=3600)   
def account_node_id(db,account_number):
    return  db.query(models.SlcMember.node_id).filter(
        models.SlcMember.member_id == models.SlcRadAccount.member_id,
        models.SlcRadAccount.account_number == account_number).scalar()

@cache.cache('get_member_node_id',expire=3600)   
def member_node_id(db,member_id):
    return  db.query(models.SlcMember.node_id).filter_by(member_id = member_id).scalar()
    
@cache.cache('get_member_by_name',expire=3600)   
def get_member_by_name(db,member_name):
    return  db.query(models.SlcMember).filter_by(member_name = member_name).first()
    
@cache.cache('get_account_by_number',expire=300)   
def get_account_by_number(db,account_number):
    return  db.query(models.SlcRadAccount).filter_by(account_number = account_number).first()
    
# @cache.cache('get_online_status',expire=30)   
def get_online_status(db,account_number):
    return  db.query(models.SlcRadOnline.id).filter_by(account_number = account_number).count() > 0
    
@cache.cache('get_param_value',expire=600)   
def get_param_value(db,pname):
    return  db.query(models.SlcParam.param_value).filter_by(param_name = pname).scalar()

def get_page_data(query):
    def _page_url(page, form_id=None):
        if form_id:return "javascript:goto_page('%s',%s);" %(form_id.strip(),page)
        request.query['page'] = page
        return request.path + '?' + urllib.urlencode(request.query)        
    page = int(request.params.get("page",1))
    offset = (page - 1) * page_size
    page_data = Paginator(_page_url, page, query.count(), page_size)
    page_data.result = query.limit(page_size).offset(offset)
    return page_data

def serial_json(mdl):
    if not mdl:return
    if not hasattr(mdl,'__table__'):return
    data = {}
    for c in mdl.__table__.columns:
        data[c.name] = getattr(mdl, c.name)
    return json.dumps(data,ensure_ascii=False)










#!/usr/bin/env python
#coding:utf-8

from bottle import Bottle
from bottle import request
from bottle import response
from bottle import redirect
from bottle import static_file
from bottle import mako_template as render
from tablib import Dataset
from toughradius.console.websock import websock
from toughradius.console import models
from toughradius.console.libs import utils
from toughradius.console.base import *
from toughradius.console.admin import forms
import bottle
import datetime
from sqlalchemy import func

__prefix__ = "/ops"

app = Bottle()
app.config['__prefix__'] = __prefix__
render = functools.partial(Render.render_app,app)

###############################################################################
# user manage        
###############################################################################
                   
@app.route('/user',apply=auth_opr,method=['GET','POST'])
def user_query(db):   
    node_id = request.params.get('node_id')
    product_id = request.params.get('product_id')
    user_name = request.params.get('user_name')
    status = request.params.get('status')
    opr_nodes = get_opr_nodes(db)
    _query = db.query(
            models.SlcRadAccount,
            models.SlcMember.realname,
            models.SlcRadProduct.product_name
        ).filter(
            models.SlcRadProduct.id == models.SlcRadAccount.product_id,
            models.SlcMember.member_id == models.SlcRadAccount.member_id
        )
    if node_id:
        _query = _query.filter(models.SlcMember.node_id == node_id)
    else:
        _query = _query.filter(models.SlcMember.node_id.in_([i.id for i in opr_nodes]))
    if product_id:
        _query = _query.filter(models.SlcRadAccount.product_id==product_id)
    if user_name:
        _query = _query.filter(models.SlcRadAccount.account_number.like('%'+user_name+'%'))
    if status:
        _query = _query.filter(models.SlcRadAccount.status == status)

    if request.path == '/user':
        return render("ops_user_list", page_data = get_page_data(_query),
                       node_list=opr_nodes, 
                       product_list=db.query(models.SlcRadProduct),**request.params)
                       
permit.add_route("%s/user"%__prefix__,u"用户账号查询",u"运维管理",is_menu=True,order=0)

@app.get('/user/trace',apply=auth_opr)
def user_trace(db):   
    return render("ops_user_trace", bas_list=db.query(models.SlcRadBas))

permit.add_route("%s/user/trace"%__prefix__,u"用户消息跟踪",u"运维管理",is_menu=True,order=1)
                   
@app.get('/user/detail',apply=auth_opr)
def user_detail(db):   
    account_number = request.params.get('account_number')  
    user  = db.query(
        models.SlcMember.realname,
        models.SlcRadAccount.member_id,
        models.SlcRadAccount.account_number,
        models.SlcRadAccount.expire_date,
        models.SlcRadAccount.balance,
        models.SlcRadAccount.time_length,
        models.SlcRadAccount.flow_length,
        models.SlcRadAccount.user_concur_number,
        models.SlcRadAccount.status,
        models.SlcRadAccount.mac_addr,
        models.SlcRadAccount.vlan_id,
        models.SlcRadAccount.vlan_id2,
        models.SlcRadAccount.ip_address,
        models.SlcRadAccount.bind_mac,
        models.SlcRadAccount.bind_vlan,
        models.SlcRadAccount.ip_address,
        models.SlcRadAccount.install_address,
        models.SlcRadAccount.create_time,
        models.SlcRadProduct.product_name
    ).filter(
            models.SlcRadProduct.id == models.SlcRadAccount.product_id,
            models.SlcMember.member_id == models.SlcRadAccount.member_id,
            models.SlcRadAccount.account_number == account_number
    ).first()
    if not user:
        return render("error",msg=u"用户不存在")
    user_attrs = db.query(models.SlcRadAccountAttr).filter_by(account_number=account_number)
    return render("ops_user_detail",user=user,user_attrs=user_attrs)
    
permit.add_route("%s/user/detail"%__prefix__,u"账号详情",u"运维管理",order=1.01)

@app.post('/user/release',apply=auth_opr)
def user_release(db):   
    account_number = request.params.get('account_number')  
    user = db.query(models.SlcRadAccount).filter_by(account_number=account_number).first()
    user.mac_addr = ''
    user.vlan_id = 0
    user.vlan_id2 = 0

    ops_log = models.SlcRadOperateLog()
    ops_log.operator_name = get_cookie("username")
    ops_log.operate_ip = get_cookie("login_ip")
    ops_log.operate_time = utils.get_currtime()
    ops_log.operate_desc = u'释放用户账号（%s）绑定信息'%(account_number,)
    db.add(ops_log)

    db.commit()
    websock.update_cache("account",account_number=account_number)
    return dict(code=0,msg=u"解绑成功")
    
permit.add_route("%s/user/release"%__prefix__,u"用户释放绑定",u"运维管理",order=1.02)    

###############################################################################
# online manage      
###############################################################################
    
@app.route('/online',apply=auth_opr,method=['GET','POST'])
def online_query(db): 
    node_id = request.params.get('node_id')
    account_number = request.params.get('account_number')  
    framed_ipaddr = request.params.get('framed_ipaddr')  
    mac_addr = request.params.get('mac_addr')  
    nas_addr = request.params.get('nas_addr')  
    opr_nodes = get_opr_nodes(db)
    _query = db.query(
        models.SlcRadOnline.id,
        models.SlcRadOnline.account_number,
        models.SlcRadOnline.nas_addr,
        models.SlcRadOnline.acct_session_id,
        models.SlcRadOnline.acct_start_time,
        models.SlcRadOnline.framed_ipaddr,
        models.SlcRadOnline.mac_addr,
        models.SlcRadOnline.nas_port_id,
        models.SlcRadOnline.start_source,
        models.SlcRadOnline.billing_times,
        models.SlcRadOnline.input_total,
        models.SlcRadOnline.output_total,
        models.SlcMember.node_id,
        models.SlcMember.realname
    ).filter(
            models.SlcRadOnline.account_number == models.SlcRadAccount.account_number,
            models.SlcMember.member_id == models.SlcRadAccount.member_id
    )
    if node_id:
        _query = _query.filter(models.SlcMember.node_id == node_id)
    else:
        _query = _query.filter(models.SlcMember.node_id.in_([i.id for i in opr_nodes]))
            
    if account_number:
        _query = _query.filter(models.SlcRadOnline.account_number.like('%'+account_number+'%'))
    if framed_ipaddr:
        _query = _query.filter(models.SlcRadOnline.framed_ipaddr == framed_ipaddr)
    if mac_addr:
        _query = _query.filter(models.SlcRadOnline.mac_addr == mac_addr)
    if nas_addr:
        _query = _query.filter(models.SlcRadOnline.nas_addr == nas_addr)

    _query = _query.order_by(models.SlcRadOnline.acct_start_time.desc())
    return render("ops_online_list", page_data = get_page_data(_query),
                   node_list=opr_nodes, 
                   bas_list=db.query(models.SlcRadBas),**request.params)

permit.add_route("%s/online"%__prefix__,u"在线用户查询",u"运维管理",is_menu=True,order=2)

###############################################################################
# ticket manage        
###############################################################################

@app.route('/ticket',apply=auth_opr,method=['GET','POST'])
def ticket_query(db): 
    node_id = request.params.get('node_id')
    account_number = request.params.get('account_number')  
    framed_ipaddr = request.params.get('framed_ipaddr')  
    mac_addr = request.params.get('mac_addr')  
    query_begin_time = request.params.get('query_begin_time')  
    query_end_time = request.params.get('query_end_time')  
    opr_nodes = get_opr_nodes(db)
    _query = db.query(
        models.SlcRadTicket.id,
        models.SlcRadTicket.account_number,
        models.SlcRadTicket.nas_addr,
        models.SlcRadTicket.acct_session_id,
        models.SlcRadTicket.acct_start_time,
        models.SlcRadTicket.acct_stop_time,
        models.SlcRadTicket.acct_input_octets,
        models.SlcRadTicket.acct_output_octets,
        models.SlcRadTicket.acct_input_gigawords,
        models.SlcRadTicket.acct_output_gigawords,
        models.SlcRadTicket.framed_ipaddr,
        models.SlcRadTicket.mac_addr,
        models.SlcRadTicket.nas_port_id,
        models.SlcMember.node_id,
        models.SlcMember.realname
    ).filter(
            models.SlcRadTicket.account_number == models.SlcRadAccount.account_number,
            models.SlcMember.member_id == models.SlcRadAccount.member_id
    )
    if node_id:
        _query = _query.filter(models.SlcMember.node_id == node_id)
    else:
        _query = _query.filter(models.SlcMember.node_id.in_([i.id for i in opr_nodes]))
    if account_number:
        _query = _query.filter(models.SlcRadTicket.account_number.like('%'+account_number+'%'))
    if framed_ipaddr:
        _query = _query.filter(models.SlcRadTicket.framed_ipaddr == framed_ipaddr)
    if mac_addr:
        _query = _query.filter(models.SlcRadTicket.mac_addr == mac_addr)
    if query_begin_time:
        _query = _query.filter(models.SlcRadTicket.acct_start_time >= query_begin_time)
    if query_end_time:
        _query = _query.filter(models.SlcRadTicket.acct_stop_time <= query_end_time)

    _query = _query.order_by(models.SlcRadTicket.acct_start_time.desc())
    return render("ops_ticket_list", page_data = get_page_data(_query),
               node_list=opr_nodes,**request.params)

permit.add_route("%s/ticket"%__prefix__,u"上网日志查询",u"运维管理",is_menu=True,order=3)

###############################################################################
# ops log manage        
###############################################################################

@app.route('/opslog',apply=auth_opr,method=['GET','POST'])
def opslog_query(db): 
    operator_name = request.params.get('operator_name')
    query_begin_time = request.params.get('query_begin_time')  
    query_end_time = request.params.get('query_end_time')  
    keyword = request.params.get('keyword')
    opr_nodes = get_opr_nodes(db)
    _query = db.query(models.SlcRadOperateLog).filter(
        models.SlcRadOperateLog.operator_name == models.SlcOperator.operator_name,
    ) 
    if operator_name:
        _query = _query.filter(models.SlcRadOperateLog.operator_name == operator_name)
    if keyword:
        _query = _query.filter(models.SlcRadOperateLog.operate_desc.like("%"+keyword+"%"))        
    if query_begin_time:
        _query = _query.filter(models.SlcRadOperateLog.operate_time >= query_begin_time+' 00:00:00')
    if query_end_time:
        _query = _query.filter(models.SlcRadOperateLog.operate_time <= query_end_time+' 23:59:59')
    _query = _query.order_by(models.SlcRadOperateLog.operate_time.desc())
    return render("ops_log_list", 
        node_list=opr_nodes,
        page_data = get_page_data(_query),**request.params)


permit.add_route("%s/opslog"%__prefix__,u"操作日志查询",u"运维管理",is_menu=True,order=4)

###############################################################################
# ops log manage        
###############################################################################

def default_start_end():
    day_code = datetime.datetime.now().strftime("%Y-%m-%d")
    begin = datetime.datetime.strptime("%s 00:00:00"%day_code,"%Y-%m-%d %H:%M:%S")
    end = datetime.datetime.strptime("%s 23:59:59"%day_code,"%Y-%m-%d %H:%M:%S")
    return time.mktime(begin.timetuple()),time.mktime(end.timetuple())

@app.get('/online/stat',apply=auth_opr)
def online_stat_query(db): 
    return render(
        "ops_online_stat",
        node_list=get_opr_nodes(db),
        node_id=None,
        day_code=utils.get_currdate()
    )

@app.route('/online/statdata',apply=auth_opr,method=['GET','POST'])
def online_stat_data(db):    
    node_id = request.params.get('node_id')
    day_code = request.params.get('day_code')  
    opr_nodes = get_opr_nodes(db)
    if not day_code:
        day_code = utils.get_currdate()
    begin = datetime.datetime.strptime("%s 00:00:00"%day_code,"%Y-%m-%d %H:%M:%S")
    end = datetime.datetime.strptime("%s 23:59:59"%day_code,"%Y-%m-%d %H:%M:%S")
    begin_time,end_time = time.mktime(begin.timetuple()),time.mktime(end.timetuple())
    _query = db.query(models.SlcRadOnlineStat)
    
    if node_id:
        _query = _query.filter(models.SlcRadOnlineStat.node_id == node_id)
    else:
        _query = _query.filter(models.SlcRadOnlineStat.node_id.in_([i.id for i in opr_nodes]))
    
    _query = _query.filter(
        models.SlcRadOnlineStat.stat_time >= begin_time,
        models.SlcRadOnlineStat.stat_time <= end_time,
    )
    _data = [ (q.stat_time*1000,q.total) for q in _query ]
    return dict(code=0,data=[{'data':_data}])
        
permit.add_route("%s/online/stat"%__prefix__,u"在线用户统计",u"运维管理",is_menu=True,order=5)


@app.get('/flow/stat',apply=auth_opr)
def online_stat_query(db): 
    return render(
        "ops_flow_stat",
        node_list=get_opr_nodes(db),
        node_id=None,
        day_code=utils.get_currdate()
    )


@app.route('/flow/statdata',apply=auth_opr,method=['GET','POST'])
def flow_stat_data(db):    
    node_id = request.params.get('node_id')
    day_code = request.params.get('day_code')    
    opr_nodes = get_opr_nodes(db)
    if not day_code:
        day_code = utils.get_currdate()
    begin = datetime.datetime.strptime("%s 00:00:00"%day_code,"%Y-%m-%d %H:%M:%S")
    end = datetime.datetime.strptime("%s 23:59:59"%day_code,"%Y-%m-%d %H:%M:%S")
    begin_time,end_time = time.mktime(begin.timetuple()),time.mktime(end.timetuple())
    _query = db.query(models.SlcRadFlowStat)
    
    if node_id:
        _query = _query.filter(models.SlcRadFlowStat.node_id == node_id)
    else:
        _query = _query.filter(models.SlcRadFlowStat.node_id.in_([i.id for i in opr_nodes]))
    
    _query = _query.filter(
        models.SlcRadFlowStat.stat_time >= begin_time,
        models.SlcRadFlowStat.stat_time <= end_time,
    )
    
    in_data = {"name":u"上行流量","data":[]}
    out_data = {"name":u"下行流量","data":[]}
    
    for q in _query:
        _stat_time = q.stat_time * 1000
        in_data['data'].append([_stat_time,float(utils.kb2mb(q.input_total))])
        out_data['data'].append([_stat_time,float(utils.kb2mb(q.output_total))])

    return dict(code=0,data=[in_data,out_data])
        
permit.add_route("%s/flow/stat"%__prefix__,u"用户流量统计",u"运维管理",is_menu=True,order=5)        


#!/usr/bin/env python
#coding=utf-8
from twisted.python import log
from toughradius.radiusd.settings import *
import datetime

def std_rate(resp,_in,_out):
    input_limit = str(_in)
    output_limit = str(_out)
    _class = input_limit.zfill(8) + input_limit.zfill(8) + output_limit.zfill(8) + output_limit.zfill(8)
    resp['Class'] = _class
    return resp

def ros_rate(resp,_in,_out):
    _irate = _in/1024/8
    _orate = _out/1024/8
    resp['Mikrotik-Rate-Limit'] = '%sk/%sk'%(_irate,_orate)
    return resp
    
def aikuai_rate(resp,_in,_out):
    _irate = _in/1024/8
    _orate = _out/1024/8
    resp['RP-Upstream-Speed-Limit'] = _irate
    resp['RP-Downstream-Speed-Limit'] = _orate
    return resp

def cisco_rate(resp,_in,_out):
    return resp

def radback_rate(resp,_in,_out):
    return resp
    
def h3c_rate(resp,_in,_out):
    return resp
    
def zte_rate(resp,_in,_out):
    resp['ZTE-Rate-Ctrl-Scr-Up'] = _in/1024
    resp['ZTE-Rate-Ctrl-Scr-Down'] = _out/1024
    return resp
    
def huawei_rate(resp,_in,_out):
    resp = std_rate(resp,_in,_out)
    resp['Huawei-Input-Average-Rate'] = _in
    resp['Huawei-Input-Peak-Rate'] = _in
    resp['Huawei-Output-Average-Rate'] = _out
    resp['Huawei-Output-Peak-Rate'] = _out
    return resp

rate_funcs = {
    '0' : std_rate,
    '9' : cisco_rate,
    '2011' : huawei_rate,
    '2352' : radback_rate,
    '3902' : zte_rate,
    '25506' : h3c_rate,
    '14988' : ros_rate,
    '10055' : aikuai_rate
}

def process(req=None,resp=None,user=None,radiusd=None,**kwargs):
    store = radiusd.store
    product = store.get_product(user['product_id']) 
    if not product:return resp
    input_limit = product['input_max_limit']
    output_limit = product['output_max_limit']
    if input_limit == 0 and output_limit == 0:
        return std_rate(resp,0,0)

    _vendor = req.vendor_id or '0'
    return rate_funcs[_vendor](resp,input_limit,output_limit)
    






#!/usr/bin/env python
#coding=utf-8
import json

def process(req=None,admin=None,**kwargs):
    msg_id = req.get("msg_id") 
    nas_addr = req.get("nas_addr") 
    acct_session_id = req.get("acct_session_id")
    message_type  =req.get("message_type")

    if not nas_addr or not acct_session_id:
        reply = {'msg_id':msg_id,'code':1,'data':u"nas_addr and acct_session_id Does not allow nulls"}
        return admin.sendMessage(json.dumps(reply),False)

    coa_client = admin.radiusd.coa_clients.get(nas_addr)
    if not coa_client:
        reply = {'msg_id':msg_id,'code':1,'data':u"CoA Client instance not exists for %s"%nas_addr}
        return admin.sendMessage(json.dumps(reply),False)

    online = admin.radiusd.store.get_online(nas_addr,acct_session_id)
    if not online:
        reply = {'msg_id':msg_id,'code':1,'data':u"online not exists"}
        return admin.sendMessage(json.dumps(reply),False)

    if message_type == 'disconnect':
        attrs = {
            'User-Name' : online['account_number'],
            'Acct-Session-Id' : acct_session_id,
            'NAS-IP-Address' : nas_addr,
            'Framed-IP-Address' : online['framed_ipaddr']
        }
        dmeq = coa_client.createDisconnectPacket(**attrs)
        coa_client.sendCoA(dmeq)
        reply = {'msg_id':msg_id,'code':0,'data':u"disconnect message send"}
        admin.sendMessage(json.dumps(reply),False)

